#include <Application.h>
#include "Application1.h"
#include "Stringtable.h"
#include "Defines.h"
#include <MediaFormats.h>
#include <fs_attr.h>
#include "PipeCmd.h"
#include <MediaFile.h>
#include <MediaTrack.h>
#include <FilePanel.h>
#include "URLView.h"
#include <be/app/Roster.h>
#include <AppFileInfo.h>
#include "M3UList.h"
#include "ColumnTypes.h"
#include "FolderRow.h"
#include "AudioRow.h"
#include "BitmapMenuItem.h"
#include "AboutWindow.h"

void printArguments(char *title, const char **args) {
	int i=0;
	printf("HELIOS: %s\n", title);
	while (args[i]!=NULL) {
		printf("\targument(%d)=%s\n", i+1, args[i]);
		i++;
	}
	printf("\n");
}



bool Application1::FileExists(BEntry *entry) {
	return entry->Exists();
}

bool Application1::FileExists(BEntry entry) {
	return FileExists(&entry);
}

bool Application1::FileExists(BFile *file) {
	return (file->InitCheck()!=B_ENTRY_NOT_FOUND);
}

bool Application1::FileExists(char *filename) {
	BEntry *entry=new BEntry(filename, true);
	bool	exists=entry->Exists();
	entry->Unset();
	delete entry;
	return exists;	
}

bool Application1::FileExists(const char *filename) {
	return FileExists((char *)filename);
}

// check, whether there is a project loaded or not
bool Application1::IsProject(bool show_alert=true) {
		if (!FileExists(BEntry(base_folder.String(), true))) {
			if (show_alert) {
				BAlert *alert=new BAlert("no project",TNOPROJECT,TOK);
				alert->Go();
			}
			return false;
		} else 
		return true;
}



Application1::Application1():BApplication("application/x-HeliosBurner.application") {

	devices=(const char **)malloc(sizeof(char *)*10);
	devicepaths=(const char **)malloc(sizeof(char *)*10);
	cdrecord_priority=100;
	filename_col=0;
	save_and_quit=false;
	is_all_showing=false;
	currentproj=(projectStruct *)malloc(sizeof(projectStruct));
	window1 = new Window1();
	window2 = new Window2();
	window3 = new Window3();
	window4 = new Window4();
	window5 = new Window5();
	settingswindow=new SettingsWindow(BRect(50,50,400,335),TSETTINGSWINDOWTITLE);
	cdrecord_output=new ProtocolWindow(BRect(0,0,350,450), "cdrecord", B_DOCUMENT_WINDOW, -1);
	mkisofs_output=new ProtocolWindow(BRect(0,0,350,450), "mkisofs", B_DOCUMENT_WINDOW, -1);
	if (!GetCDRDevices()) {
		BAlert *alert=new BAlert("alert",TNODEVICESFOUND,TOK);
		alert->Go();
		be_app->Quit();
	}
	ReadSettings();
	Update_DATA_AUDIO();
	if (window1->Lock()) {
		window1->view1->UpdatePanels();
		window1->Unlock();
	}
	window1->Show();
	window4->Lock();
	if (settingswindow->readerPUM->CountItems()<2) 
		settingswindow->twodrivecopyCB->SetEnabled(false); else
		settingswindow->twodrivecopyCB->SetEnabled(true);
	window4->Unlock();
}


Application1::~Application1(void) {
	
	free(devices);
	free(currentproj);
}


void
Application1::ReadyToRun(void) {
	// set file type database entry for Helios project files
	app_info	appinfo;
	BBitmap	*licon, *micon;
	
	be_app->GetAppInfo(&appinfo);
	BAppFileInfo	*appfileinfo=new BAppFileInfo(new BFile(&appinfo.ref, B_READ_WRITE));
	BMimeType	*projectmime=new BMimeType("application/x-vnd.Helios-project");

	licon=new BBitmap(BRect(0,0,31,31), B_CMAP8);
	appfileinfo->GetIcon(licon, B_LARGE_ICON);
	projectmime->SetIconForType("application/x-vnd.Helios-project",licon, B_LARGE_ICON);
	projectmime->SetIcon(licon, B_LARGE_ICON);
	micon=new BBitmap(BRect(0,0,15,15), B_CMAP8);
	appfileinfo->GetIcon(micon, B_MINI_ICON);
	projectmime->SetIconForType("application/x-vnd.Helios-project",micon, B_MINI_ICON);
	projectmime->SetIcon(micon, B_MINI_ICON);
	projectmime->SetPreferredApp(appinfo.signature);
	projectmime->SetLongDescription("Helios CDRECORDer Project File");
	projectmime->SetShortDescription("Helios Project");
//	if (projectmime->IsInstalled()) projectmime->Delete();
	projectmime->Install();
	
	delete projectmime;	
	delete appfileinfo;
	delete licon;
	delete micon;

}


void Application1::CheckIfProjectChanged(void) {

	if (currentproj->_filesystem!=window1->view1->GetFilesystem()) has_project_changed=true;
	if (BString(currentproj->_volumename).Compare(window1->view1->GetVolumeName())!=0) has_project_changed=true;
	if (BString(currentproj->_publisher).Compare(settingswindow->publisherTC->Text())!=0) has_project_changed=true;
	if (BString(currentproj->_preparer).Compare(settingswindow->preparerTC->Text())!=0) has_project_changed=true;
	if (BString(currentproj->_application).Compare(settingswindow->applicationTC->Text())!=0) has_project_changed=true;
	if ((currentproj->_cdtype==0) && (window1->view1->CDType()==AUDIOCD_INDEX)) has_project_changed=true;
};



bool
Application1::QuitRequested(void) {

	CheckIfProjectChanged();
	while ((save_and_quit) && (has_project_changed)) {
		snooze(50000);
 	};
	
	if ((has_project_changed) && (IsProject(false))) {
		BAlert *alert=new BAlert("alert",TASKFORSAVINGCHANGES,TYES, TNO, TCANCEL);
		switch (alert->Go()) {
			case 0: {
				be_app->PostMessage(SAVEPROJECT_MIMSG);
				save_and_quit=true;
				return false;
			}
			
			case 1: {
				break;
			}
			
			case 2: {
				return false;
			}
		}
	}




	SaveSettings();
	while (CountWindows()>0) {WindowAt(0)->Lock();WindowAt(0)->Quit();}
	return true;
}


void
Application1::MessageReceived(BMessage *message) {
	
	switch (message->what)
	{
		
		case B_SIMPLE_DATA:
		{
		uint32 		type;
		int32 		count;
		off_t		filesize=0;
		BEntry		*entry=new BEntry();
		entry_ref	ref;
		BPath		path;
		char		mp3name[250];

		if (!IsProject()) break;

		window1->Lock();
		message->GetInfo("refs", &type, &count);
		if (type != B_REF_TYPE) { delete entry; break;}

		//printf("HELIOS adding files now\n");
		watch=new BStopWatch("fileadder", true);
		addershows=false;

		for (int32 i = --count; i >= 0; i--) 
		{
			if (message->FindRef("refs", i, &ref) != B_OK)
				{
				if (window1->Alert("Failure at BMessage.FindRef()\n\nProbably you didn't drop a file.")==true) {HideStatus(); delete entry; delete watch; break;}
				} else
				entry->SetTo(&ref,true);

	   		   if (entry->GetPath(&path)==B_OK)
			   	{
	     		           	sprintf(mp3name,"%s",path.Leaf());
    			 	if (entry->GetSize(&filesize)==B_OK)
	      		 		{
						{
			 		 		cdir=current_folder;
			 		 		if (window1->view1->CDType()==AUDIOCD_INDEX)
				 		 		has_project_changed=true;
			 	 			if (CreateEntry((char *)path.Path(), (char *)path.Leaf()))
			 	 			window1->size+=filesize;



							if (window1->view1->CDType()==AUDIOCD_INDEX) {
								BEntry entry;
								float size;			

					
								entry.SetTo(path.Path(),true);
								MMediaInfo *info=new MMediaInfo(entry);
								size=info->Size();
								if (size>0)
									window1->view1->listview1->AddRow(new AudioRow(window1->view1->listview1->iCountItems()+1, path.Leaf(), 0, size, size/176400));
								delete info;
								entry.Unset();
							}




			 	 		}
    					}
		   		}
		   }
		   path.Unset();
		   entry->Unset();
		delete entry;
		delete watch;
		 ShowFolder();
		HideStatus();	
		window1->Unlock();

		break;
		}
		
		case 'mm01': // update display
		{
		char temp[10];
		window1->Lock();
		window1->Update_sizes();
		directories=0;
		files=0;
		GetFolder(BDirectory(base_folder.String()));
		sprintf(temp,"%ld",files);
		window1->view1->box2->filecounterSV->SetText(temp);
		sprintf(temp,"%ld",directories);
		window1->view1->box2->foldercounterSV->SetText(temp);
		window1->Unlock();
		break;
		}
		
		case PREFERENCES_MIMSG:
		{
		settingswindow->Lock();
		if (settingswindow->IsHidden())
		settingswindow->Show();
		settingswindow->Unlock();
		break;
		}
		
		case DELETE_FILE_MSG: // remove selected items
		{
		BEntry *file=new BEntry();
		//off_t filesize;
		BString str="", filename="";
		
		
		watch=new BStopWatch("fileadder", true);
		addershows=false;
		window1->Lock();
		  for (int32 i=0; i<window1->view1->listview1->iCountItems();i++)
 			 {
			if (!addershows)
				if (watch->ElapsedTime()>1200000)
						{
						ShowStatus(TREMOVINGFILES);
						addershows=true;
						}
			  if (window1->view1->listview1->IsSelected(i)) 
			     {
			    // BStringItem *si=(BStringItem *)window1->view1->listview1->ItemAt(i);
			    if (window1->view1->CDType()==AUDIOCD_INDEX) filename=((AudioRow *)(window1->view1->listview1->RowAt(i)))->GetFilename();
			    if (window1->view1->CDType()==DATACD_INDEX) filename=((FolderRow *)(window1->view1->listview1->RowAt(i)))->GetFilename();
			    if (window1->view1->CDType()==BOOTABLECD_INDEX) filename=((FolderRow *)(window1->view1->listview1->RowAt(i)))->GetFilename();
			     
			     
	     			if (filename.Compare("..")!=0)
	     			//if (!((current_folder.Compare(base_folder)==0) && (i==0)))
				{	     		
				     //str="";
				     str.SetTo(current_folder.String()).Append(filename.String());
				     //printf("REMOVE %s\n",str.String());
	
				     if (file->SetTo(str.String(),false)==B_OK)
				    	 {
					if (file->IsDirectory())
						{
						str.SetTo("rm -r \"").Append(current_folder).Append(filename.String()).Append("\"");
						system(str.String());			     	
						} else
						file->Remove();
					file->Unset();
					// there has been something changed
					has_project_changed=true;
				    	 }
				    str.SetTo(filename.String());
				    window1->view1->listview1->RemoveRow(i);

				    // if audio mode, check for item with the same name, because these are
				    // splitted parts and have to be removed, too!
				    if (window1->view1->CDType()==AUDIOCD_INDEX) {
				    	for (int j=0; j<window1->view1->listview1->iCountItems();j++) {
				    		if (str.Compare(((AudioRow *)(window1->view1->listview1->RowAt(j)))->GetFilename())==0) {
				    			window1->view1->listview1->RemoveRow(j);
				    			j=-1;
				    		}
				    	}

				    }
				     i=-1;
				}
   			    }
  			}
		delete watch;
		delete file;
		if (addershows) HideStatus();
		UpdateTrackNumbers();
		Update_path();
		RefreshStatusView();
		window1->Unlock();
		break;
		}
		
		case MOVE_UP_MSG:
		{
		MoveUp();
		UpdateTrackNumbers();
		break;
		}
		
		case MOVE_DOWN_MSG:
		{
		MoveDown();
		UpdateTrackNumbers();
		break;
		}
		
		case 'mm40': {
			settingswindow->Lock();
			if (settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())
					==settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())) 
			settingswindow->twodrivecopyCB->SetEnabled(false); else
			settingswindow->twodrivecopyCB->SetEnabled(true);
			settingswindow->Unlock();	
			break;
		}

		case 'mm03': // doppelklick auf listitem
		{
		// auf audio-datei geklickt?
		if (window1->view1->CDType()==AUDIOCD_INDEX) break;
		
		// clv update
		/*window1->Lock();
		window1->view1->listview1->Refresh();
		window1->Unlock();*/
		
		FolderRow *cs=(FolderRow *)window1->view1->listview1->CurrentSelection();
		
		if (cs==NULL) break;
		if (!cs->IsFolder()) break;
		//printf("ENTER FROM %s\t%s\n",current_folder.String(),si->Text());
		BEntry entry;
		BString str="";
		str=current_folder;
		if (BString(cs->GetFilename()).Compare("..")==0)
			{
			if (str[str.Length()-1]=='/') str.RemoveLast("/");
			str.Remove(str.FindLast("/"),str.Length()-str.FindLast("/"));
			}
		else
			str.Append(cs->GetFilename());
		entry.SetTo(str.String());
		if (entry.IsDirectory())
			{
			if (str[str.Length()-1]!='/') str.Append("/");
			current_folder=str;
			//printf("ENTERED %s\t%s\n",current_folder.String(),si->Text());
			}
		entry.Unset();
		ShowFolder();
		Update_path();
		break;
		}
		
		case 'mm04': // Liste updaten
		{
		ShowFolder();
		Update_path();
		sprintf(settings._heliospath,settingswindow->heliospathTC->Text());
		if (settings._heliospath[strlen(settings._heliospath)-1]!='/')
			sprintf(settings._heliospath, "%s/", settings._heliospath);
		if (settingswindow->Lock()) {
			settingswindow->heliospathTC->SetText(settings._heliospath);
			settingswindow->Unlock();
		}
				
//		if (window1->view1->radiobutton1->Value()==B_CONTROL_OFF) {
//			RemoveFileTrackNumbers();
//			AddFileTrackNumbers();
//		}
		break;
		}
	
		case 'mm05': // Pathdisplay updaten
		{
		//ShowFolder();
		Update_path();
		break;
		}
		
		case 'mm07':
		{
		ShowStatus(TCHECKINGDEVICES);
		if (GetCDRDevices())
			{
			Update_DATA_AUDIO();
			} else
			{
			BAlert *alert=new BAlert("alert",TNODEVICESFOUND,TOK);
			alert->Go();
			be_app->Quit();
			}
		HideStatus();
		break;
		}
		
		case PAUSECHANGE_MSG:
		{
		ChangePauses();
		break;
		}

		case DATAUDIO_MSG:
		{
		window1->Lock();
		window1->view1->UpdatePanels();
		window1->Unlock();
		Update_DATA_AUDIO();
		ShowFolder(true);		// update audio, too!
		Update_path();
		window1->Lock();
		window1->Unlock();
		break;
		}
	
		case UPDATEINFO_MSG: {
			if (!IsProject()) break;
			ShowStatus(TCALCCDSIZE);
			GetImageSize();
			char temp[10];
			window1->Lock();
			directories=0;
			files=0;
			GetFolder(BDirectory(base_folder.String()));
			sprintf(temp,"%ld",files);
			window1->view1->box2->filecounterSV->SetText(temp);
			sprintf(temp,"%ld",directories);
			window1->view1->box2->foldercounterSV->SetText(temp);
			window1->Unlock();
			ShowFolder();
			Update_path();
			HideStatus();
			break;
		}
	
		case BURN_MSG:
		{
		if (!IsProject()) break;
		if (!CheckCDSize()) break;
		window5->Lock();
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON)
			window5->SetTitle(TSTATUSWINDOWTITLESIMULATION); else
			window5->SetTitle(TSTATUSWINDOWTITLE);
		window5->Unlock();
		
		switch(window1->view1->CDType()) {
			case	BOOTABLECD_INDEX:
			case	DATACD_INDEX:
				if ((settingswindow->streamingCB->Value()==B_CONTROL_OFF)
						||
					/* bfs? */ (window1->view1->GetFilesystem()==2))
					MakeImage_NOSTREAMING();
				else
				if (settingswindow->streamingCB->Value()==B_CONTROL_ON)
					MakeImage();
				break;
				
			case	AUDIOCD_INDEX:
				BurnAudio();
				break;
			default:	break;
		}

		//HideStatus();
		window5->Lock();
		window5->SetTitle(TSTATUSWINDOWTITLE);
		window5->Unlock();
		break;
		}

		case FILESYSTEM_MSG:
		{
	//	GetImageSize();
		break;
		}
		
		case RENAME_FILE_MSG:
		{
		FolderRow *selected=(FolderRow *)window1->view1->listview1->CurrentSelection();
		if (selected==NULL) break;
		window2->Lock();
		window2->todo=1;
		window2->SetTitle(TRENAMEITEM);
		window2->view2->textcontrol3->SetLabel(TNEWNAME);
		window2->view2->textcontrol3->SetText(selected->GetFilename());
		window2->Unlock();
		window2->Show();
		break;
		}
	
		case NEW_FOLDER_MSG:
		{
		if (!IsProject()) break;
		window2->Lock();
		window2->todo=2;
		window2->SetTitle(TCREATEFOLDER);
		window2->view2->textcontrol3->SetLabel(TFOLDERNAME);
		window2->view2->textcontrol3->SetText("");
		window2->Unlock();
		window2->Show();
		break;
		}


		case SET_INDICES_MSG:
		{
		window2->Lock();
		window2->todo=3;
		window2->SetTitle(TINDICES);
		window2->view2->textcontrol3->SetLabel(TINDICESLIST);
		AudioRow *selected=(AudioRow *)window1->view1->listview1->CurrentSelection();
		window2->view2->textcontrol3->SetText(selected->GetIndexList());
		window2->Unlock();
		window2->Show();
		break;
		}
		
		case 'mm13': // indexliste erstellen
		{
		BString str;
		window1->Lock();
		str.SetTo(window2->view2->textcontrol3->Text());
		AudioRow *selected=(AudioRow *)window1->view1->listview1->CurrentSelection();
		selected->SetIndexList(str.String());
		has_project_changed=true;
		window1->Unlock();
		break;
		}


		case SETCDTEXTTITLE_MSG:
		{
		window2->Lock();
		window2->todo=5;
		window2->SetTitle(TCDTEXTTITLE);
		window2->view2->textcontrol3->SetLabel(T___CDTEXTTITLE);
		AudioRow *selected=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		window2->view2->textcontrol3->SetText(selected->GetCDTitle());
		window2->Unlock();
		window2->Show();
		break;
		}
		
		case 'mm15': // cd-text title
		{
		BString str;
		window1->Lock();
		str.SetTo(window2->view2->textcontrol3->Text());
		AudioRow *selected=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		selected->SetCDTitle(str.String());
		has_project_changed=true;
		window1->Unlock();
		break;
		}


		case CDRECORDPROTOCOL_MIMSG: {
			if (cdrecord_output->Lock()) {
				if (cdrecord_output->IsHidden())
				cdrecord_output->Show();
				cdrecord_output->Unlock();
			}
			break;
		}

		case MKISOFSPROTOCOL_MIMSG: {
			if (mkisofs_output->Lock()) {
				if (mkisofs_output->IsHidden())
				mkisofs_output->Show();
				mkisofs_output->Unlock();
			}
			break;
		}
		
		
		case 'mm11': // markierten eintrag umbenennen
		{
		BEntry file;
		//off_t filesize;
		BString str="";
		FolderRow *selected=(FolderRow *)(window1->view1->listview1->CurrentSelection());
		
		if (selected==NULL) return;
		window1->Lock();
		if (BString(selected->GetFilename()).Compare("..")!=0)
		{	     		
			     str="";
			     str << current_folder.String() << selected->GetFilename();
			     //printf("WANNA REMOVE %s\n",str.String());
			     if (file.SetTo(str.String(),false)==B_OK)
			    	 {
			    	 str="";
			    	 str << current_folder.String() << window2->view2->textcontrol3->Text();
			    	 file.Rename(str.String(),false);
			    	 selected->SetFilename(window2->view2->textcontrol3->Text());
				has_project_changed=true;
	//		    	 window1->view1->listview1->InvalidateItem(selected);
			    	 }
		}
		file.Unset();
		//window1->view1->listview1->ItemAt(selected)->DrawItem(window1->view1, window1->view1->listview1->ItemFrame(selected),true);
		Update_path();
		window1->Unlock();
		break;
		}
		
		
		
		case 'mm12': // neuen ordner erstellen
		{
	//	BEntry file;
		//off_t filesize;
		BString str="";
		window1->Lock();
		str.Append(current_folder.String()).Append(window2->view2->textcontrol3->Text());
		BDirectory newdir;
		newdir.CreateDirectory(str.String(),NULL);
		has_project_changed=true;
		ShowFolder();
		window1->Unlock();
		break;
		}
		
		case NEWPROJECT_MIMSG: {
			NewProject();
			currentproject="";
			ShowFolder();
			if (window1->Lock()) {
				window1->view1->SetCDType(currentproj->_cdtype);
				window1->view1->listview1->SetListMode(currentproj->_cdtype);
				window1->Unlock();
			}
//			ShowFolderPanel();			
			break;
		}

		case OPENPROJECT_MIMSG: {
			ShowOpenPanel();
			break;
		}

		case SAVEPROJECTAS_MIMSG: {
			if (!IsProject()) break;
			ShowSavePanel();
			break;
		}


		case REMOVEPROJECT_MIMSG: {
			if (!IsProject()) break;
			BString		bf=base_folder;
			if (bf[bf.Length()-1]=='/') bf.RemoveLast("/");
			BEntry	*folder_of_project=new BEntry(bf.String(), false);
			BEntry	*projectfile=new BEntry(currentproject.String(), true);

			BAlert *alert=new BAlert("erase project",TAREYOUSURE,TERASE, TMOVETOTRASH, TCANCEL);
			switch (alert->Go()) {
				case 0: {
					DeleteAllIn(BDirectory(bf.String()));
					folder_of_project->Remove();
					projectfile->Remove();
					currentproject="";
					base_folder="";
					current_folder="";
					ShowFolder();
					window1->SetTitle(TWINDOWTITLE);
					break;
				}
				
				case 1: {
					folder_of_project->MoveTo(new BDirectory("/boot/home/Desktop/Trash"));
					projectfile->MoveTo(new BDirectory("/boot/home/Desktop/Trash"));
					currentproject="";
					base_folder="";
					current_folder="";
					ShowFolder();
					window1->SetTitle(TWINDOWTITLE);
					break;
				}
				
				case 2: {
					break;
				}
			};
			folder_of_project->Unset();
			delete folder_of_project;
			projectfile->Unset();
			delete projectfile;
			break;
		}


		case B_CANCEL: {
			save_and_quit=false;
			break;
		}

		case OPEN_FPMSG: {	
			entry_ref ref;
			if (message->FindRef("refs",  &ref) != B_OK) break;
			
			BEntry *entry=new BEntry(&ref, true);
			LoadProject(entry);
			ShowFolder(true);
			if (window1->Lock()) {
				window1->view1->SetCDType(currentproj->_cdtype);
				window1->Unlock();
			}
			entry->Unset();
			delete entry;

			break;
		}

		case FOLDER_FPMSG: {	
			entry_ref ref;
			BPath	path;
			if (message->FindRef("refs",  &ref) != B_OK) break;
			
			BEntry *entry=new BEntry(&ref, true);
			entry->GetPath(&path);
		
			BDirectory HeliosDir;
			BString bf=path.Path();
			if (bf[bf.Length()-1]=='/') bf.RemoveLast("/");
			if (HeliosDir.SetTo(path.Path())!=B_OK)
				HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
			HeliosDir.Unset();
			base_folder=path.Path();
			if (base_folder[base_folder.Length()-1]!='/') base_folder.Append("/");
			current_folder=base_folder;
			NewProject();
			
			ShowFolder();
			entry->Unset();
			delete entry;

			break;
		}

		case SAVE_FPMSG: {	
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			message->FindString("name",&n);
			name.Append(n.String());
			SaveProject(new BEntry(name.String(), true));
			if (save_and_quit) be_app->PostMessage(B_QUIT_REQUESTED);
			en.Unset();
			path.Unset();
			break;
		}

		case SAVEPROJECT_MIMSG: {
			if (!IsProject()) break;
			BEntry entry(currentproject.String(), true);
			
			if (!entry.Exists())
				ShowSavePanel();
			else {
				SaveProject(&entry);
				if (save_and_quit) be_app->PostMessage(B_QUIT_REQUESTED);
			}
			entry.Unset();
			break;
		}
		
		case STANDARDPROJECT_MIMSG: {
			if (!IsProject()) break;
			SetAsStandardProject();
			break;
		}

		case FASTBLANK_MIMSG:
		{
		BlankRW("-blank=fast");
		break;
		}

		case COMPLETEBLANK_MIMSG:
		{
		BlankRW("-blank=all");
		break;
		}

		case UNCLOSESESSION_MIMSG:
		{
		BlankRW("-blank=unclose");
		break;
		}

		case BLANKSESSION_MIMSG:
		{
		BlankRW("-blank=session");
		break;
		}
		
		case EXPORT_M3U_MIMSG: {
			if (!IsProject()) break;
			ShowExportPanel(new BMessage(EXPORT_M3U_FPMSG));
			break;
		}
		
		case EXPORT_M3U_FPMSG: {
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			message->FindString("name",&n);
			name.Append(n.String());
			ExportAsM3U(new BEntry(name.String(), true));
			en.Unset();
			path.Unset();
			break;
		}
		
		case EXPORT_APML_MIMSG: {
			if (!IsProject()) break;
			ShowExportPanel(new BMessage(EXPORT_APML_FPMSG));
			break;
		}
		
		case EXPORT_APML_FPMSG: {
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			message->FindString("name",&n);
			name.Append(n.String());
			ExportAsAPML(new BEntry(name.String(), true));
			en.Unset();
			path.Unset();
			break;
		}

		case EXPORT_FILELIST_MIMSG: {
			if (!IsProject()) break;
			ShowExportPanel(new BMessage(EXPORT_FILELIST_FPMSG));
			break;
		}
		
		case EXPORT_FILELIST_FPMSG: {
			entry_ref ref;
			BString name="";
			BString n="";
			BEntry	en;
			BPath path;
			
			if (message->FindRef("directory",  &ref) != B_OK) break;
			en.SetTo(&ref, true);
			en.GetPath(&path);
			name.SetTo(path.Path());
			if (name.ByteAt(name.Length()-1)!='/') name.Append("/");
			message->FindString("name",&n);
			name.Append(n.String());
			ExportAsFileList(new BEntry(name.String(), true));
			en.Unset();
			path.Unset();
			break;
		}


		case FOLDERMENU_MSG: {	// user clicked on statusview's folders-popupmenu
			BString folder;
			if (message->FindString("folder", &folder)==B_OK) {
				if (folder[folder.Length()-1]!='/') folder.Append("/");
				current_folder=folder;
				ShowFolder();
				Update_path();
			}
			break;
		}

		case IMAGE_MSG: 
		{
		window4->Lock();
		window4->SetWorkspaces(B_CURRENT_WORKSPACE);
		if (window4->Lock()) {
			if (window4->IsHidden()) window4->Show();
			window4->Unlock();
		}
		window4->Unlock();
		break;
		}
		
		case 'mm41': {
		
			switch(donow) {
			
				case 1: {	// BURN IMAGE FILE
					BEntry entry;
					entry.SetTo(window4->view4->textcontrol4->Text(),false);
					if (!entry.Exists()) {
						BAlert *alert=new BAlert("no file",TNOTEXIST_ALERT,TOK);
						alert->SetShortcut(0,B_ENTER);
						alert->Go();
					} else
						BurnImage(BString(window4->view4->textcontrol4->Text()));
					entry.Unset();
					break;
				}
				
				case 2: {	// READ CD IMAGE
					BEntry entry;
					entry.SetTo(window4->view4->textcontrol4->Text(),false);
					if (entry.Exists())
						{
						BAlert *alert=new BAlert("replace file",TREPLACE_ALERT,TNO,TYES);
						alert->SetShortcut(0,B_ESCAPE);
						if (alert->Go()==1) entry.Remove(); else break;
						}
					ReadImage(BString(window4->view4->textcontrol4->Text()));
					entry.Unset();
					break;
				}

				case 3: {	// CREATE FILE IMAGE
					BEntry entry;
					entry.SetTo(window4->view4->textcontrol4->Text(),false);
					if (entry.Exists())
						{
						BAlert *alert=new BAlert("replace file",TREPLACE_ALERT,TNO,TYES);
						alert->SetShortcut(0,B_ESCAPE);
						if (alert->Go()==1) entry.Remove(); else break;
						}
					MakeImageOnly(BString(window4->view4->textcontrol4->Text()));
					entry.Unset();
					break;
				}
			}
			
			break;
		}
		
		case BURNIMAGEFILE_MIMSG:
		{
		if (window4->Lock()) {
			if (window4->IsHidden()) window4->Show();
			donow=1;
			window4->Unlock();
		}
		break;
		}
		
		case READCDIMAGE_MIMSG:
		{
		if (window4->Lock()) {
			if (window4->IsHidden()) window4->Show();
			donow=2;
			window4->Unlock();
		}
		break;
		}

		case COPYDATACD_MIMSG: {
		if (settingswindow->twodrivecopyCB->Value()==B_CONTROL_ON)
			CopyOnTheFly(); else
			CopyDataCD();
		break;
		}
		
		case COPYAUDIOCD_MIMSG: {
		int track_count=0;
		if (settingswindow->twodrivecopyCB->Value()==B_CONTROL_ON)
			CopyOnTheFly(); else
			track_count=ReadAudioCD();
			printf("track_count=%d\n",track_count);
			ShowFolder(true);
		break;
		}

		case CREATEFILEIMAGE_MIMSG: {
		if (!IsProject()) break;
		if (window4->Lock()) {
			if (window4->IsHidden()) window4->Show();
			window4->Unlock();
			donow=3;
		}
		break;
		}

		case BURNSPEED_MSG:
		{
		BString str=TRECORDERSPEED;
		str << " (" << settingswindow->writerspeedSL->Value()*2 << "x)";
		settingswindow->Lock();
		settingswindow->writerspeedSV->SetText(str.String());
		settingswindow->Unlock();
		Calculate_time();
		break;
		}
		
		case SPLIT_TRACK_MSG:
		{
		BString str="";
		window2->Lock();
		window2->todo=4;
		window2->SetTitle(TSPLITTRACK);
		window2->view2->textcontrol3->SetLabel(TSPLITOFFSET);
		AudioRow *selected=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		// display the half size of track as a suggestion for the split offset
		str << selected->GetLength()/2 << "s";
		window2->view2->textcontrol3->SetText(str.String());
		window2->Unlock();
		window2->Show();
		break;
		}
		
		case 'mm14': // splits the selected track into two parts
		{
		AudioRow *ai=(AudioRow *)(window1->view1->listview1->CurrentSelection());
		BString str=window2->view2->textcontrol3->Text();
		size_t splitoffset; // in bytes
		
		if (str.FindFirst("f")==B_ERROR) {
			// typed in as seconds
			str.RemoveAll("s");
			splitoffset=atol(str.String())*176400;			
		} else {
			// typed in as frames
			str.RemoveAll("f");
			splitoffset=176400*atol(str.String())/75;
		}
		window1->Lock();
		window1->view1->listview1->AddRow(new AudioRow(ai->GetTrackNumber()+1, ai->GetFilename(),0,ai->GetBytes()-splitoffset, (ai->GetBytes()-splitoffset)*176400), window1->view1->listview1->IndexOf(ai)+1);
		ai->SetBytes(splitoffset);
		if (!ai->IsSplitted()) {ai->SetSplitIndex(0);ai->SetSplitted(true);}
		has_project_changed=true;
		UpdateTrackNumbers();
		window1->Unlock();
		break;
		}

		case FIXATE_CHKBOX_MSG:
		{
		Calculate_time();
		break;
		}
		
		
		default:
		{
			BApplication::MessageReceived(message);
			break;
		}	
	}

}

void
Application1::RefsReceived(BMessage *message)
{

	uint32 		type;
	int32 		count;
	BEntry		*entry=new BEntry();
	entry_ref	ref;

	message->GetInfo("refs", &type, &count);
	
	if (type != B_REF_TYPE) { delete entry; return;}
	if (message->FindRef("refs", 0, &ref) != B_OK) {
		if (window1->Alert("Failure at BMessage.FindRef()\n\nProbably you didn't drop a project file.")==true) { delete entry; return;}
	} else	{
		if (entry->SetTo(&ref,true)==B_OK) {
			LoadProject(entry);
		}
	}
	delete entry;	
	ShowFolder();
}

void Application1::AboutRequested(void) {
	app_info	info;
	AboutWindow	*aboutW=new AboutWindow(BRect(100,100,400,280), "About");
	BScreen	*screen=new BScreen();

	aboutW->MoveTo((int)((screen->Frame().Width()-aboutW->Bounds().Width())/2), (int)((screen->Frame().Height()-aboutW->Bounds().Height())/2));
	delete screen;
	if (GetAppInfo(&info)==B_OK) {
		BBitmap bmp(BRect(0,0,31,31), B_CMAP8);
		if (BNodeInfo::GetTrackerIcon(&info.ref, &bmp, B_LARGE_ICON)==B_OK)
			aboutW->SetIcon(&bmp);
	}
	aboutW->SetApplicationName(ABOUT_APPLICATIONNAME);
	aboutW->SetVersionNumber(ABOUT_VERSIONNUMBER);
	aboutW->SetCopyrightString(ABOUT_COPYRIGHTSTRING);
	aboutW->SetText(ABOUT_TEXT);
	aboutW->Show();
}




BBitmap *GetIcon(entry_ref *ref) {

	BBitmap *icon=new BBitmap(BRect(0.0,0.0,15.0,15.0), B_CMAP8);
	BEntry	*entry=new BEntry(ref, true);
	entry_ref	r;
	
	entry->GetRef(&r);
	delete entry;
		
	BNodeInfo::GetTrackerIcon(&r, icon, B_MINI_ICON);
	return icon;
}


/****************************************************************************
 *
 *	Shows content of current folder in file window.
 *
 ***************************************************************************/
void
Application1::ShowFolder(bool audio) {
	BDirectory dir;
	BEntry de;
	BPath path;
	int tracks=0;
	entry_ref	ref;
	
	
	dir.SetTo(current_folder.String());
	dir.Rewind();
	window1->Lock();
	if ((audio) || ((window1->view1->CDType()==DATACD_INDEX) || (window1->view1->CDType()==BOOTABLECD_INDEX))) {
//		window1->view1->listview1->iMakeEmpty();
		while (window1->view1->listview1->CountRows()>0) {
			BRow	*row=window1->view1->listview1->RowAt(0);
			window1->view1->listview1->BColumnListView::RemoveRow(row);
			delete row;
		}
	}
	if (current_folder!=base_folder)
		window1->view1->listview1->AddRow(new FolderRow("..", true, NULL, true),(int32)0);
			while (dir.GetNextEntry(&de, false)==B_OK) {
				if (de.IsDirectory()) {
					de.GetPath(&path);
					if ((window1->view1->CDType()==DATACD_INDEX) ||
						(window1->view1->CDType()==BOOTABLECD_INDEX)) {	
							BFile file(&de, B_READ_ONLY);
							off_t filesize;
							file.GetSize(&filesize);
							de.GetRef(&ref);
							window1->view1->listview1->AddRow(new FolderRow(path.Leaf(), 0, true, GetIcon(&ref)),(int32)0);
							file.Unset();
					}
				}
			}
			dir.Rewind();
	
			while (dir.GetNextEntry(&de,false)==B_OK)
			{
			if ((de.IsSymLink()) || (de.IsFile()))
				{
				de.GetPath(&path);
				if ((window1->view1->CDType()==DATACD_INDEX) ||
					(window1->view1->CDType()==BOOTABLECD_INDEX))
					{
					BFile file(&de, B_READ_ONLY);
					off_t filesize;
					if (file.GetSize(&filesize)!=B_OK) filesize=0;
					de.GetRef(&ref);
					window1->view1->listview1->AddRow(new FolderRow(path.Leaf(),filesize, false, GetIcon(&ref)), window1->view1->listview1->CountRows());
					file.Unset();
					} else
					if (audio)
					{
					BEntry entry;
					float size;

					
					entry.SetTo(path.Path(),true);
					MMediaInfo *info=new MMediaInfo(entry);
					size=info->Size();
					//info.~MMediaInfo();

					if (size>0)
					window1->view1->listview1->AddRow(new AudioRow(++tracks, path.Leaf(),0,size, size/176400));
					delete info;
					entry.Unset();
					
					}
				//printf("FILEFOUND %s\n",path.Leaf());
				}

			}
dir.Unset();
de.Unset();
path.Unset();

RefreshStatusView();

if (window1->view1->listview1->CountRows()>0)
	window1->view1->listview1->ScrollTo(window1->view1->listview1->RowAt(0));
PopulateFoldersMenu();
window1->Unlock();
}


void Application1::RefreshStatusView() {
	int32 items=window1->view1->listview1->CountRows();
	if (current_folder!=base_folder) items--;
	if (window1->Lock()) {
		window1->view1->listview1->statusView->SetNumberOfItems(items);
		window1->Unlock();
	}
}




//fügt rekursiv den ordner/die datei hinzu
bool Application1::CreateEntry(char *pathname, char *entryname)
{
BEntry entry, de;
BString str="";
BPath path;
BDirectory d;
if (!addershows)
	if (watch->ElapsedTime()>1200000)
						{
						ShowStatus(TADDINGFILES);
						addershows=true;
						}

entry.SetTo(pathname, false); //links nicht nachverfolgen
if (entry.IsDirectory())
	{
	BDirectory dir;
	if (dir.SetTo(&entry)!=B_OK) {entry.Unset(); return false;}
	if (cdir[cdir.Length()-1]!='/') cdir.Append("/");
	cdir.Append(entryname);
	//printf("CDIR %s\n",cdir.String());

	if (dir.CreateDirectory(cdir.String(),NULL)!=B_OK) {entry.Unset(); dir.Unset();return false;}
	dir.Rewind();
			while (dir.GetNextEntry(&de,false)==B_OK)
			{
			de.GetPath(&path);
			str="";
			if (de.IsDirectory())
				{
			//	de.GetPath(&path);
			//	str="";
				if (cdir[cdir.Length()-1]!='/') cdir.Append("/");
				str.Append(cdir).Append(path.Leaf());
				//printf("CREATEENTRY %s\n",path.Path());
				CreateEntry((char *)path.Path(), (char *)path.Leaf());
				cdir.Remove(cdir.FindLast("/"),cdir.Length()-cdir.FindLast("/"));
				//printf("PARENTDIR %s\n",cdir.String());
				}
			if (de.IsFile())
				{
			//	de.GetPath(&path);
			//	str="";
				//if (cdir[cdir.Length()-1]!='/') cdir.Append("/");
				str.Append(cdir).Append(path.Leaf());
				//printf("CREATEENTRY %s\n",path.Path());
				CreateEntry((char *)path.Path(), (char *)path.Leaf());
				}

			}
	} else
if (entry.IsFile())
	{
	entry.GetPath(&path);
	str=cdir;
	BSymLink	*sl=new BSymLink(&entry);
	d.CreateSymLink(str.Append("/").Append(path.Leaf()).String(), pathname, sl);
	delete sl;
	//str << "ln -s \"" << pathname << "\" \"" << cdir << "\"";
	//printf("%s\n",str.String());
	//system(str.String());
	}

//window1->Lock();
//window1->view1->stringview1->SetText("");
//window1->Unlock();
entry.Unset();
de.Unset();
path.Unset();
d.Unset();

return true;
}


// unused by now
void Application1::Update_path() {
/*
	int32 selected=window1->view1->listview1->CurrentSelection();
	BString str=current_folder;
	str.RemoveFirst(base_folder);
	pathdisplay.SetTo(window1->view1->textcontrol1->Text());
	pathdisplay.Prepend("/").Append("/").Append(str.String());
	
	if ((selected>=0) && (window1->view1->listview1->CurrentSelection(1)<0)) {
		BStringItem *si=(BStringItem *)window1->view1->listview1->ItemAt(selected);
		if (window1->view1->radiobutton1->Value()==B_CONTROL_ON)
			pathdisplay.Append(si->Text()); else
			pathdisplay.SetTo(si->Text());
		
		window1->Lock();
		window1->view1->stringview1->SetText(pathdisplay.String());
		window1->Unlock();
	} else {
		window1->Lock();
		if (window1->view1->radiobutton1->Value()==B_CONTROL_ON)
			window1->view1->stringview1->SetText(pathdisplay.String()); 
		else
			window1->view1->stringview1->Clear();
		window1->Unlock();
	}
*/
}

// this is Philippe Houdoin's
status_t get_device_path_target_lun
	(
	const char *	dev,
	int *		 	path,
	int *		 	target,
	int *			lun
	)
{
	int p, t, l;

	p = t = l = 0;
	if (!strncmp("/dev/disk/scsi/", dev, 15))
		sscanf(dev, "/dev/disk/scsi/%d/%d/%d/raw", &p, &t, &l);
	else if ( ! strncmp("/dev/disk/ide/ata/", dev, 18))
		{
		char	ide_targ[64];
		
		sscanf(dev, "/dev/disk/ide/ata/%d/%s/%d/raw", &p, ide_targ, &l);
		p += 8;
		if ( strncmp( ide_targ, "master", 6) == 0 )	t = 0;
		if ( strncmp( ide_targ, "slave", 5) == 0 ) t = 1;
		}
	else if ( ! strncmp("/dev/disk/ide/atapi/", dev, 18)) 
		{
		char	ide_targ[64];

		sscanf(dev, "/dev/disk/ide/atapi/%d/%s/%d/raw", &p, ide_targ, &l);
		p += 8;
		if ( strncmp( ide_targ, "master", 6) == 0 )	t = 0;
		if ( strncmp( ide_targ, "slave", 5) == 0 ) t = 1;
		};

	*path 	= p;
	*target = t;
	*lun	= l;
	return B_OK;
}


// this is Philippe Houdoin's
void Application1::lookup_devices(const char *path)
{
	BDirectory dir(path);

	if (dir.InitCheck() != B_OK)
		return;
		
	BEntry entry;

	dir.Rewind();
	while (dir.GetNextEntry(&entry) >= 0)
		{
		BPath name;
		entry.GetPath(&name);
		if (entry.IsDirectory())
			lookup_devices(name.Path());

		if ( strcmp(name.Leaf(), "raw") != 0 )
			continue;
			
		int 				fd, e;
		int 				path, targ, lun;
		char  				ide_targ[64];
		const char * 		dev;
		uchar 				data[36], sense[16];
		int					type;
		raw_device_command 	rdc;

		dev = name.Path();
		path = targ = lun = 0;
		if (!strncmp("/dev/disk/scsi/", dev, 15))
			sscanf(dev,"/dev/disk/scsi/%d/%d/%d/raw",&path, &targ, &lun);
		else if ( ! strncmp("/dev/disk/ide/ata/", dev, 18))
			{
			sscanf(dev, "/dev/disk/ide/ata/%d/%s/%d/raw", &path, ide_targ, &lun);
			if ( strncmp( ide_targ, "master", 6) == 0 )	targ = 0;
			if ( strncmp( ide_targ, "slave", 5) == 0 ) targ = 1;
			}
		else if ( ! strncmp("/dev/disk/ide/atapi/", dev, 18)) 
			{
			sscanf(dev, "/dev/disk/ide/atapi/%d/%s/%d/raw", &path, ide_targ, &lun);
			if ( strncmp( ide_targ, "master", 6) == 0 )	targ = 0;
			if ( strncmp( ide_targ, "slave", 5) == 0 ) targ = 1;
			};

		// fill out our raw device command data
		rdc.data = data;
		rdc.data_length = 36;
		rdc.sense_data = sense;
		rdc.sense_data_length = 0;
		rdc.timeout = 1000000;
		rdc.flags = B_RAW_DEVICE_DATA_IN;
		rdc.command_length = 6;
		rdc.command[0] = 0x12;
		rdc.command[1] = 0x00;
		rdc.command[2] = 0x00;	
		rdc.command[3] = 0x00;	
		rdc.command[4] = 36;
		rdc.command[5] = 0x00;	

		if ((fd = open(dev,0)) < 0)
			continue;
				
		e = ioctl(fd, B_RAW_DEVICE_COMMAND, &rdc, sizeof(rdc));
		close(fd);
		if ((e != 0) ||
			(rdc.cam_status != CAM_REQ_CMP))
			{
			// printf("B_RAW_DEVICE_COMMAND(inquiry) failed on %s!\n", dev);
			continue;
			};

		type = data[0] & 0x1F;
		if ( type != 5 ) // if not CD-ROM type
			continue;

	//	BMenuItem *	item;
		char 		vendor[32];
		char 		product[32];
		char		devnumber[7];
	//	BMessage * 	msg;
			
		BString 	label;
			
	//	msg = new BMessage(msg_what);
	//	msg->AddString("path", dev);
//		printf("%d,%d,%d\n",path, targ, lun);	
		get_device_path_target_lun(dev,&path,&targ,&lun);
		sprintf(devnumber,"%d,%d,%d",path, targ, lun);	
		devices[number]=strdup(devnumber);
		devicepaths[number++]=strdup(dev);
		sprintf(vendor, "%8.8s", data + 8);
		//printf("VENDOR: %s\n",vendor);
		sprintf(product, "%16.16s", data + 16);
		//printf("PRODUCT: %s\n",product);
		
		label << vendor;
		label.RemoveAll(" ");
		//label.Capitalize();

		label << " " << product; 

		settingswindow->writerPUM->AddItem(
			new BMenuItem(label.String(),new BMessage('mm40')) );
		settingswindow->readerPUM->AddItem(
			new BMenuItem(label.String(),new BMessage('mm40')) );
		
	//	item = new BMenuItem(label.String(), msg);
	//	menu->AddItem(item);
//		name.Unset();
		};	// while	
	dir.Unset();
	entry.Unset();
}




bool Application1::GetCDRDevices()
{
number=0;
			SCSI_IDE=" (SCSI)";
			lookup_devices("/dev/disk/scsi");
			SCSI_IDE=" (IDE)";
			lookup_devices("/dev/disk/ide");

// return false, if no cd-writers found
if (settingswindow->writerPUM->CountItems()==0) return false;
settingswindow->writerPUM->ItemAt(0)->SetMarked(true);
settingswindow->readerPUM->ItemAt(0)->SetMarked(true);
return true;
}

bool Application1::GetImageSize()
{
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString filesystem="";
		int in, out, err;
		
		// audio cd
		if (window1->view1->CDType()==AUDIOCD_INDEX)
			{
			float time=0;
			//pause=atof(window1->view1->textcontrol2->Text());
	
			for (int i=0; i<window1->view1->listview1->CountRows();i++)
			{
			AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
				time+=si->GetLength();
				time+=(float)si->GetPregap()/75.0;
			}
			window1->Lock();
			window1->size=(size_t)time*153600;
			window1->Update_sizes();
			Calculate_time();
			window1->Unlock();

			return true;
			}
			
		// BFS-Image???
		if (window1->view1->GetFilesystem()==2) {
			
			imgsize=0;
			files=0;
			directories=0;
		
			// get size of directories and files
			GetSize(BDirectory(base_folder.String()));
			imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
									// positionen der icons in den fenstern
									// werden auch abgespeichert...
			imgsize+=(5144*1024)+64*imgsize/(1024*1024)+384;
			window1->Lock();
			window1->size=(off_t)imgsize;
			window1->Update_sizes();
			Calculate_time();
			window1->Unlock();
			return true;
		}
			
		
//		window1->Lock();
//		window1->view1->stringview1->SetText("Calculating image size... please wait.");
//		window1->Unlock();
		args = (const char **)malloc(sizeof(char *) * (53));
		arguments=MakeMKISOFSCommandline(args, "", true);

		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,10);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image(mkisofs)\n\nUnknown load error. Sorry.")==true) 
						 {free(args); return false;}
		rename_thread(Thread,MKISOFSTHREADNAME);
		
		size_t bytesread=0;
		char buffer[10000];
		bool gotit=false;

		resume_thread(Thread);
		do {
			bytesread=read(err,(void *)buffer, 10000);
			buffer[bytesread]=0;

			// output it to the log-window
			mkisofs_output->AddText(buffer);

			if (bytesread>0) {
				if (strstr(buffer,"to be written =")) {
					window1->Lock();
					BString s;
					s=strstr(buffer,"to be written =");
					s.RemoveFirst("to be written =");
					window1->size=atoll(s.String())*2048;
					//window1->view1->stringview1->Clear();
					window1->Update_sizes();
					Calculate_time();
					gotit=true;
					window1->Unlock();
				} // strstr			
			}
			snooze(100000);
		} while ((Thread==find_thread(MKISOFSTHREADNAME)) || (!gotit));
		close(in);close(out);close(err);
			
		free(args);
	return true;
}

/*
 *	Thread function for data transport mkisofs <-> cdrecord
 */
typedef struct {
	int in;
	int out;
	size_t buffersize;
	bool *working;
} bufferfunc;

int32 DataBuffer(void *data) {
	
	char *buffer(NULL);
	size_t written=0;
	size_t bytesread=10, byteswritten=0;
	bufferfunc *d=(bufferfunc *)data;
	buffer=(char *)malloc(d->buffersize);
	
	while ((d->working) && (written>=0) && (bytesread>0)) {
		bytesread=read(d->in,buffer, d->buffersize);
		if (bytesread>0) {
			written=write(d->out,buffer,bytesread);
			if (written>0) byteswritten+=written;
		}
	}
	
	free(buffer);
	printf("HELIOS: %ld bytes redirected.\n", byteswritten);
	return 0;
}

status_t Application1::WaitForDisc(const char *device) {

		char 	info[4];
		int	timeout=DEFAULTTIMEOUT*10;
		
		ShowStatus2(TWAITINGFORDISC,0);
		while (DeviceStatus(device)!=E_NO_ERROR) {
		snooze(100000);
		timeout--;
		if ((timeout==0) || (window5->interrupted)) {
			HideStatus2();
			return B_TIMEOUT;
		}

		sprintf(info,"%d\"",timeout/10);
		SetStatus2Info((const char *)info);
		SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*10));
		};
		return B_OK;
}



void Application1::MakeImage()
{
		int arguments=0;
		thread_id buffer_Thread, mkisofs_Thread, cdrecord_Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		char *info;
		bool working;
		bufferfunc	bf;
		int mkisofs_in, mkisofs_out, mkisofs_err, cdrecord_in, cdrecord_out, cdrecord_err;
		int32 start_sec, end_sec;
		bool	multi_flag=false;

		// wait till a disc is in the drive and drive is ready
		if (WaitForDisc(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())])==B_TIMEOUT) return;

		//									//
		//   begin to produce the options for mkisofs...	//
		//									//

		GetImageSize();
		device << devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())];
		args = (const char **)malloc(sizeof(char *) * (42));
		// Make MKISOFS options
		
		arguments=MakeMKISOFSCommandline(args, "");
	
		// multisession disc selected???
		if (window1->view1->IsMultisession()) {
			// ask whether this session should be the first, last or another
			// session of the disc
			BAlert *alert=new BAlert("error",MULTISESSION_ALERT,MULTISESSION_ALERT_B0,MULTISESSION_ALERT_B1,MULTISESSION_ALERT_B2);
			int selected=alert->Go();
		
			multi_flag=true;
			if (selected!=2)
				if (GetMultisessionSectors(start_sec, end_sec)!=B_OK) {
					BAlert *alert=new BAlert("error",MULTISESSION_FAILS_ALERT,TOK);
					alert->Go();			
					free(args);
					HideStatus2();
					return;
				}
		
			if (selected==0) {		// it is another session!
				args[arguments++] = strdup("-M");
				args[arguments++] = strdup(devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
				args[arguments++] = strdup("-C");
				filesystem << start_sec << "," << end_sec;
				args[arguments++] = strdup(filesystem.String());
			}

			if (selected==1) {		// it is the last session!
				args[arguments++] = strdup("-M");
				args[arguments++] = strdup(devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
				args[arguments++] = strdup("-C");
				filesystem << start_sec << "," << end_sec;
				args[arguments++] = strdup(filesystem.String());
				// leave the "-multi"-option for cdrecord!
				multi_flag=false;
			}
		
		}
		args[arguments]=NULL;
		//printArguments("MKISOFS Arguments",args);


		// launch mkisofs thread now
		mkisofs_Thread=pipe_command(arguments, args, mkisofs_in, mkisofs_out, mkisofs_err);
		set_thread_priority(mkisofs_Thread,MKISOFS_PRIORITY);
			if (mkisofs_Thread<0)
				if (window1->Alert("Failure at: load_image(mkisofs)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return;}
		SetStatus2(TWAITING,mkisofs_Thread);
		rename_thread(mkisofs_Thread,MKISOFSTHREADNAME);

		// Make CDRECORD options
		arguments=0;
		args[arguments++] = strdup("/boot/home/config/bin/cdrecord");
		args[arguments++] = strdup(device.String());
		if (settingswindow->burnproofCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("driveropts=burnproof");
		args[arguments++] = strdup("-v");
		if (settingswindow->allowoverburningCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-overburn");
		args[arguments++] = strdup("-waiti");
		args[arguments++] = strdup("-speed=");
		str.SetTo("");
		str << (settingswindow->writerspeedSL->Value())*2;
		args[arguments++] = strdup(str.String());
		if (!window1->view1->IsMultisession())
			if (settingswindow->fixateCB->Value()==B_CONTROL_OFF) args[arguments++] = strdup("-nofix");
		if (multi_flag) args[arguments++] = strdup("-multi");
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-dummy");
		str.SetTo("");
		str << window1->size;
		args[arguments++] = strdup("tsize=");
		args[arguments++] = strdup(str.String());
		args[arguments++] = strdup("-data");
		args[arguments++] = strdup("-");
		args[arguments]=NULL;
		
		printArguments("cdrecord streaming args",args);

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
			if (cdrecord_Thread<0)
				if (window1->Alert("Failure at: load_image(cdrecord)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return;}
		SetStatus2(TWAITING,cdrecord_Thread);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);
		working=true;
		bf.working=&working;
		bf.buffersize=4*1024;
		bf.in=mkisofs_out;
		bf.out=cdrecord_in;

		buffer_Thread=spawn_thread(DataBuffer,"Buffer Thread",TORAW_PRIORITY,(void *)(&bf));
		rename_thread(buffer_Thread,BUFFERTHREADNAME);
		resume_thread(buffer_Thread);	
		resume_thread(mkisofs_Thread);	
		resume_thread(cdrecord_Thread);	





		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		int track=1;
		float length=0;	// in MB
		float op=0,progressed=0;
		char *info2;
		BString s;

		mkisofs_output->Start(mkisofs_err);
		do {

			bytesread=read(cdrecord_out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			
			cdrecord_output->AddText(buffer);
			
			
			if (bytesread>0) {
				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					asprintf(&info,"%dx (%0.1f MB/s)",speed,176400.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				
				// Performing OPC...
				if (strstr(buffer,"Performing OPC...")) {
					SetStatus2(TPERFORMINGOPC,-1);
				}

				// finished fixating...
				if (strstr(buffer,"Fixating time:")) {
					SetStatus2(TFINISHEDWRITING,-1);
				}

				// start burning...
				if (strstr(buffer,"Starting new track")) {
					SetStatus2(TBURNINGDATACD,-1);
				}

				// Fixating...
				if (strstr(buffer,"Fixating...")) {
					SetStatus2(TFIXATING,-1);
					SetPercentage(100.0);
				}

				// get track number
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.RemoveFirst("Track");
					s.Remove(s.FindFirst(":"),s.Length()-s.FindFirst(":"));
					track=atoi(s.String());
				}		

				// get track length
				//if (length==0)
				if (strstr(buffer,"of")) {
					s=strstr(buffer,"of");
					s.RemoveFirst("of");
					s.Remove(s.FindFirst("MB"),s.Length()-s.FindFirst("MB"));
					length=atof(s.String());
				}		
	
				// get write position
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.Remove(0,9);
					s.Remove(s.FindFirst("of"),s.Length()-s.FindFirst("of"));
					progressed=atof(s.String());
					if (op<progressed) {
						op=progressed;
						if (length>0) {
							asprintf(&info,"%0.1f%%",100.0*(float)progressed/(float)length);
							asprintf(&info2,"%s %d",TTRACK,track);
							SetStatus2Info(info, info2);
							SetPercentage(100.0*(float)progressed/(float)length);
						}
					}
				}		
			}
			snooze(30000);
		}while (cdrecord_Thread==find_thread(CDRECORDTHREADNAME));
		mkisofs_output->Stop();
		working=false;
		bf.working=&working;snooze(100000);
		if (find_thread(BUFFERTHREADNAME)==buffer_Thread) kill_thread(buffer_Thread);
		if (find_thread(MKISOFSTHREADNAME)==mkisofs_Thread) kill_thread(mkisofs_Thread);
		close(mkisofs_in);close(mkisofs_out);close(mkisofs_err);
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);
		if (window5->interrupted) {HideStatus2(); free(args); return;}
			
		if (settingswindow->ejectwriterCB->Value()==B_CONTROL_ON)
			eject_media(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
		free(args);
		HideStatus2();
}


void Application1::MakeImage_NOSTREAMING()
{
		BString temp=settings._heliospath;
		temp.Append(DEFAULTIMAGEFILENAME);
		// BeOS image?
		if (window1->view1->GetFilesystem()!=2)
		{ //no
		MakeMKISOImage(temp.String());
		} else
		{
		MakeBFSImage(temp.String());
		}	 
		BurnImage(temp.String());	
}



int Application1::ReadAudioCD() {
		const char **args;
		int arguments=0;
		thread_id Thread;
		BString str="";
		int in, out, err;
				
		// allocate some memory for the command line array
		args = (const char **)malloc(sizeof(char *) * (23));

		// fill array with arguments
		args[arguments++] = strdup("/boot/home/config/bin/cdda2wav");
		args[arguments++] = strdup("-D");
		args[arguments++] = strdup(devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
		args[arguments++] = strdup("-x");
		args[arguments++] = strdup("-B");
		args[arguments++] = strdup("-t");
		args[arguments++] = strdup("1");
		str=base_folder;
		str.Append("Track");
		args[arguments++] = strdup(str.String());
		args[arguments] = NULL;


		// and _please_ launch a new thread!
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,16);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image(cdda2wav)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return -1;}
		ShowStatus2(TREADINGAUDIO,Thread);
		rename_thread(Thread,CDDA2WAVTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		char *info, *info2;
		float percentage, oldpercentage=1000;
		BString	per="";
		int	trackno=0;
		
		while (Thread==find_thread(CDDA2WAVTHREADNAME)) {
			bytesread=read(err,(void *)buffer, 1000);
			buffer[bytesread]=0;
			if (bytesread>0) {
				if (strstr(buffer,"%")) {
					per=buffer;
					per.Remove(per.FindFirst("%"), per.Length()-per.FindFirst("%"));
					while (per.Length()>3) per.Remove(0,1);
					percentage=atof(per.String());
					if (percentage<oldpercentage) trackno++;
					asprintf(&info,"%0.0f%%", percentage);
					asprintf(&info2,"%s %d",TTRACK, trackno);
					SetStatus2Info(info, info2);
					SetPercentage(percentage);
					oldpercentage=percentage;
				} // strstr			
			}
			snooze(70000);
		}
		close(in);close(out);close(err);
		if (window5->interrupted) {HideStatus2(); free(args); return -1;}
			
		free(args);
		HideStatus2();
		return trackno;
}



bool Application1::BurnImage(BString filename) {
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		BString fixate="";
		int in, out, err;
		char *info, *info2;

		// wait till a disc is in the drive and drive is ready
	
		window5->Lock();
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON)
			window5->SetTitle(TSTATUSWINDOWTITLESIMULATION); else
			window5->SetTitle(TSTATUSWINDOWTITLE);
		window5->Unlock();

		if (!FileExists(filename.String())) return false;
		
		ShowStatus2(TWAITINGFORDISC,0);
		if (WaitForDisc(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())])==B_TIMEOUT) return false;

		device << devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())];
		str="";
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup("/boot/home/config/bin/cdrecord");
		args[arguments++] = strdup(device.String());
		device.SetTo("-speed=");
		device << (settingswindow->writerspeedSL->Value())*2;
		args[arguments++] = strdup(device.String());
		if (settingswindow->burnproofCB->Value()==B_CONTROL_ON) args[arguments++]=strdup("driveropts=burnproof");
		if (settingswindow->fixateCB->Value()==B_CONTROL_OFF) args[arguments++] = strdup("-nofix");
		if (settingswindow->allowoverburningCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-overburn");
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-dummy");
		if (window1->view1->IsMultisession()) args[arguments++] = strdup("-multi");
		args[arguments++] = strdup("-data");
		args[arguments++] = strdup("-v");
		args[arguments++] = strdup(filename.String());
		args[arguments]=NULL;
		
		printArguments("cdrecord non-streaming args",args);
		
		
		
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,cdrecord_priority);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image(cdrecord)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return false;}
		SetStatus2(TWAITING,Thread);
		rename_thread(Thread,CDRECORDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		int track=1;
		float length=0;	// in MB
		float op=0,progressed=0;
		BString s;
		
		while (Thread==find_thread(CDRECORDTHREADNAME)) {
			bytesread=read(out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			
			
			cdrecord_output->AddText(buffer);
			
			
			if (bytesread>0) {

				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					asprintf(&info,"%dx (%0.1f MB/s)",speed,176400.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				
				// Performing OPC...
				if (strstr(buffer,"Performing OPC...")) {
					SetStatus2(TPERFORMINGOPC,-1);
				}

				// finished fixating...
				if (strstr(buffer,"Fixating time:")) {
					SetStatus2(TFINISHEDWRITING,-1);
				}

				// start burning...
				if (strstr(buffer,"Starting new track")) {
					SetStatus2(TBURNINGDATACD,-1);
				}

				// Fixating...
				if (strstr(buffer,"Fixating...")) {
					SetStatus2(TFIXATING,-1);
				}

				// get track number
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.RemoveFirst("Track");
					s.Remove(s.FindFirst(":"),s.Length()-s.FindFirst(":"));
					track=atoi(s.String());
				}		

				// get track length
				if (length==0)
				if (strstr(buffer,"of")) {
					s=strstr(buffer,"of");
					s.RemoveFirst("of");
					s.Remove(s.FindFirst("MB"),s.Length()-s.FindFirst("MB"));
					length=atof(s.String());
				}		
	
				// get write position
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.Remove(0,9);
					s.Remove(s.FindFirst("of"),s.Length()-s.FindFirst("of"));
					progressed=atof(s.String());
					if (op<progressed) {
						op=progressed;
						if (length>0) {
							asprintf(&info,"%0.1f%%",100.0*(float)progressed/(float)length);
							asprintf(&info2,"%s %d",TTRACK,track);
							SetStatus2Info(info, info2);
							SetPercentage(100.0*(float)progressed/(float)length);
						}
					}
				}		
			}
			snooze(10000);
		}
		close(in);close(out);close(err);
		window5->Lock();
			window5->SetTitle(TSTATUSWINDOWTITLE);
		window5->Unlock();
		if (window5->interrupted) {HideStatus2(); free(args); return false;}
			
		if (settingswindow->ejectwriterCB->Value()==B_CONTROL_ON)
			eject_media(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
		
		BAlert *alert=new BAlert("delete imagefile",TDELETEIMAGEFILE,TYES,TNO);
		alert->SetShortcut(1,B_ESCAPE);
		str="";
		str.SetTo("rm \"");
		str << filename.String() << "\"";
		
		if (alert->Go()==0) system(str.String());
		HideStatus2();
		free(args);
return true;
}









void Application1::ShowStatus(const char *text)
{
		window3->Lock();
		window3->view3->stringview12->SetText(text);
		window3->MoveTo(window1->Frame().left+(window1->Bounds().Width()-window3->Bounds().Width())/2,
					window1->Frame().top+(window1->Bounds().Height()-window3->Bounds().Height())/2);
		window3->SetWorkspaces(B_CURRENT_WORKSPACE);
		if (window3->IsHidden()) window3->Show();
		window3->Unlock();
}


void Application1::SetStatus(const char *text)
{
		window3->Lock();
		window3->view3->stringview12->SetText(text);
		window3->Unlock();
}



void Application1::HideStatus()
{
		window3->Lock();
		if (!window3->IsHidden()) window3->Hide();
		window3->Unlock();
}




void Application1::ShowStatus2(const char *text, thread_id Thread)
{
		window5->Lock();
		window5->Thread=Thread;
		window5->EnableControls();
		window5->interrupted=false;
		window5->view5->stringview12->SetText(text);
		window5->MoveTo(window1->Frame().left+(window1->Bounds().Width()-window5->Bounds().Width())/2,
					window1->Frame().top+(window1->Bounds().Height()-window5->Bounds().Height())/2);
		window5->SetWorkspaces(B_CURRENT_WORKSPACE);
		window5->view5->progressSB->Reset();
		if (window5->IsHidden()) window5->Show();
		window5->Unlock();
}


void Application1::SetStatus2(const char *text, thread_id Thread)
{
		window5->Lock();
		if (Thread!=-1) {
			window5->Thread=Thread;
			window5->view5->progressSB->Reset();
		}
		window5->EnableControls();
		window5->interrupted=false;
		window5->view5->stringview12->SetText(text);
		if (window5->IsHidden()) window5->Show();
		window5->Unlock();
}



void Application1::HideStatus2()
{
		window5->Lock();
		if (!window5->IsHidden()) window5->Hide();
		window5->Unlock();
}


void Application1::SetStatus2Info(const char *text, const char *leading)
{
		BString s=text;
		window5->Lock();
		if (s.Compare(window5->view5->progressSB->TrailingText())!=0)
		window5->view5->progressSB->SetTrailingText(text);
		s=leading;
		if (s.Compare(window5->view5->progressSB->Text())!=0)
		window5->view5->progressSB->SetText(leading);
		window5->Unlock();
}


void Application1::SetPercentage(float percentage) {
	window5->Lock();
	if (percentage>100) percentage=100;
	if (percentage<0) percentage=0;
	window5->view5->progressSB->Update(percentage-window5->view5->progressSB->CurrentValue());
	window5->Unlock();
}


// returns always true (very important function ;-)
bool Application1::OnlyFiles()
{
/*	BEntry f;
	BString str;

		// erst die gesamte liste durchgehen, ob auch nur audio-dateien drin sind
		
		for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			BStringItem *si=(BStringItem *)window1->view1->listview1->ItemAt(i);
			str=base_folder;
			str.Append(si->Text());
			if (f.SetTo(str.String(),true)!=B_OK) return false; // file ist jetzt der erste Listeneintrag
			if (f.IsDirectory()) {f.Unset();return false;} //es gibt keine audio-ordner...
			}
	f.Unset();
	return true;
*/
	return true;
}


void Application1::ChangePauses()
{
float pause=0;
char *bla;

window1->Lock();
pause=atof(window1->view1->textcontrol2->Text());
if (pause>4800) pause=4800;
if (pause<0) pause=0;
asprintf(&bla,"%.2f",pause);
for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
			if (window1->view1->listview1->IsSelected(i))
				{
				si->SetPregap(pause);
				//window1->view1->listview1->InvalidateRow(i);
				//si->Select();
				}
			}
window1->view1->textcontrol2->SetText("");
//window1->view1->textcontrol2->SetText(bla);
//window1->view1->textcontrol2->MakeFocus(false);
window1->Unlock();
}

void Application1::DoAudioMenu() {
	window1->Lock();

	while (window1->view1->listview1->cmenu->CountItems()>0)
		window1->view1->listview1->cmenu->RemoveItem(0L);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->deleteItem);
	window1->view1->listview1->cmenu->AddSeparatorItem();
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->moveupItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->movedownItem);
	window1->view1->listview1->cmenu->AddSeparatorItem();
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->indexItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->splitItem);
//	window1->view1->listview1->cmenu->AddSeparatorItem();
//	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->CDTEXTTitleItem);
	window1->view1->listview1->cmenu->SetTargetForItems(be_app_messenger);
	window1->Unlock();
}


void Application1::DoDataMenu() {

	if (window1->view1->listview1->cmenu==NULL) return;
	window1->Lock();

	while (window1->view1->listview1->cmenu->CountItems()>0)
		window1->view1->listview1->cmenu->RemoveItem(0L);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->deleteItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->newfolderItem);
	window1->view1->listview1->cmenu->AddItem(window1->view1->listview1->renamefileItem);
	window1->view1->listview1->cmenu->SetTargetForItems(be_app_messenger);
	window1->Unlock();
}



void Application1::Update_DATA_AUDIO()
{

	if (window1->view1->listview1->cmenu==NULL) return;
	window1->Lock();
	settingswindow->Lock();
	//window1->view1->stringview1->Clear();
		if ((window1->view1->CDType()==DATACD_INDEX) ||
			(window1->view1->CDType()==BOOTABLECD_INDEX))
			// auf DATA
			{
			window1->view1->textcontrol2->SetEnabled(false);
			//window1->view1->textcontrol1->SetEnabled(true);
			window1->view1->export_M3U->SetEnabled(false);
			window1->view1->export_APML->SetEnabled(false);
			//window1->view1->menufield2->SetEnabled(true);
			DoDataMenu();
			} else
			// auf AUDIO
		if (window1->view1->CDType()==AUDIOCD_INDEX)
			{
			current_folder=base_folder;
			window1->view1->textcontrol2->SetEnabled(true);
			//window1->view1->textcontrol1->SetEnabled(false);
			window1->view1->export_M3U->SetEnabled(true);
			window1->view1->export_APML->SetEnabled(true);
			//window1->view1->menufield2->SetEnabled(false);
			DoAudioMenu();
			}
	window1->view1->listview1->SetListMode(window1->view1->CDType());
	settingswindow->Unlock();
	window1->Unlock();
}









void  OutPutRAW(const char *filename, int fd) {
	BFile *infile=new BFile();
	char *buffer(NULL);
	const int buffersize=4*1024*1024;
	int64 framesdecoded=0;

	printf("HELIOS: writing RAW audio: %s\n", filename);
	buffer=(char *)malloc(buffersize);
	infile->SetTo(filename, B_READ_ONLY);
	do {
		framesdecoded=infile->Read((void *)buffer, buffersize);
		write(fd, &buffer,framesdecoded);
	} while (framesdecoded>0);
	free(buffer);
	infile->Unset();
	delete infile;
}





bigtime_t xyround(float f) {

	if ((float)(f-(bigtime_t)f)>0.000F) return ((bigtime_t)f)+1; else return (bigtime_t)f;
}





// Decodes compressed audio data to raw audio
bool ReadData(BMediaFile *mfile, int fd) {
	media_multi_audio_format raw;
	int64 framesRead;
	media_format mf;
	BMediaTrack *track;
	int64 j=0,frameCount;
	bigtime_t length=0;
	const int buffersize=2*1024*1024;
	char *audioData(NULL);

	if (mfile->CountTracks()!=1) return false;
	track=mfile->TrackAt(0);
		if (track->InitCheck()!=B_OK) return false;
	if (track->EncodedFormat(&mf)==B_OK)
		printf("HELIOS: audio input is %0.0fHz/%ld channels\n", mf.u.raw_audio.frame_rate, mf.u.raw_audio.channel_count);
	mf.type=B_MEDIA_RAW_AUDIO;
	mf.u.raw_audio.frame_rate=44100.0F;					// 44.1kHz
	mf.u.raw_audio.channel_count=2;						// stereo
	mf.u.raw_audio.format=raw.B_AUDIO_SHORT; 				// 16 bit
	mf.u.raw_audio.byte_order=B_MEDIA_LITTLE_ENDIAN;			// Intel
	mf.u.raw_audio.buffer_size=buffersize;
	if (track->DecodedFormat(&mf)==B_OK)
		printf("HELIOS: audio output is %0.0fHz/%ld channels\n", mf.u.raw_audio.frame_rate, mf.u.raw_audio.channel_count);
	length=xyround(2352.0*(float)track->Duration()/13333.3333333333);
	frameCount=track->CountFrames();
	
	audioData = (char*)malloc(buffersize);
	framesRead=1;
	while ((j<length) && (framesRead>0)) {
		track->ReadFrames(audioData, &framesRead);
		if (framesRead*4>length-j) framesRead=(length-j)/4;
		if (framesRead>0)
			write(fd, audioData,framesRead*4); 
		j+=framesRead*4;
	}
	for (int i=0; i<buffersize;i++) audioData[i]=0;
	while (j<length) {
		if (length-j>buffersize) framesRead=buffersize; else framesRead=length-j;
		write(fd, audioData,framesRead);
		j+=framesRead;
	}
	mfile->ReleaseAllTracks();
	free(audioData);
	return true;
}

// opens the media file
// returns true if successful, false if not
bool OpenFile(const char *filename, int fd) {
	entry_ref 	ref;
	BMediaFile 	*mfile;
	BEntry		*file=new BEntry(filename, true);
	bool		ok;
	
	if (file->InitCheck()!=B_OK) {delete file; return false;}
	if (file->GetRef(&ref)!=B_OK) {file->Unset();delete file; return false;}
	file->Unset();
	delete file; 
	mfile=new BMediaFile(&ref); 
	if (mfile->InitCheck()!=B_OK) 
		{
		OutPutRAW(filename, fd);
		delete mfile;
		return false;
		}
	ok=ReadData(mfile, fd);
	delete mfile;
	return ok;
}



typedef struct 
	audio_parameters {
		char 	*filename[99];
		int	trackcount;
		int	fd;
};

/*
 *	The following thread-function steps through the file list to process all items
 */
int32 AudioBufferProducer(void *data) {
	audio_parameters	*ap=(audio_parameters *)data;
	
	for (int i=0; i<ap->trackcount; i++) {
		printf("HELIOS filename[%d/%d]: %s\n", i,ap->trackcount, ap->filename[i]);
		OpenFile(ap->filename[i],ap->fd);
	}
	return 0;
}







void Application1::BurnAudio()
{
		int arguments=0;
		thread_id buffer_Thread, cdrecord_Thread;
		const char **args;
		BString str="";
		BString toraw="";
		BString temp="";
		BString device="dev=";
		BString endstr="";
		BString fixate="";
		BString bproof="";
//		BEntry file;
		audio_parameters	ap;
		char *info;
		int  cdrecord_in, cdrecord_out, cdrecord_err;

		// wait till a disc is in the drive and drive is ready
		if (WaitForDisc(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())])==B_TIMEOUT) return;
//		SaveCDTEXTAlbum();

		args = (const char **)malloc(sizeof(char *) * (700));
//		// Make toRAW options
//		arguments=0;
//		args[arguments++] = strdup("/boot/home/config/bin/toRAW");
//
//		//Parameter für "toRAW" erstellen
//		for (int i=0; i<window1->view1->listview1->CountItems();i++)
//			{
//			AudioItem *si=(AudioItem *)window1->view1->listview1->ItemAt(i);
//			toraw.SetTo(base_folder);
//			toraw.Append(si->Text());
//			args[arguments++] = strdup(toraw.String());
//			}
//		args[arguments]=NULL;
//
//		// launch toRAW thread now
//		toraw_Thread=pipe_command(arguments, args, toraw_in, toraw_out, toraw_err);
//		set_thread_priority(toraw_Thread,TORAW_PRIORITY);
//			if (toraw_Thread<0)
//				if (window1->Alert("Failure at: load_image(toraw)\n\nUnknown load error. Sorry.")==true) 
//						 {HideStatus2(); free(args); return;}
//		SetStatus2(TWAITING,toraw_Thread);
//		rename_thread(toraw_Thread,TORAWTHREADNAME);
//

		// Make CDRECORD options
		device << devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())];
		arguments=0;
		args[arguments++] = strdup("/boot/home/config/bin/cdrecord");
		args[arguments++] = strdup(device.String());
		if (settingswindow->burnproofCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("driveropts=burnproof");
		args[arguments++] = strdup("-v");
		if (settingswindow->allowoverburningCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-overburn");
		args[arguments++] = strdup("-waiti");
		if (settingswindow->writeDAOCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-dao");
		args[arguments++] = strdup("-speed=");
		str.SetTo("");
		str << (settingswindow->writerspeedSL->Value())*2;
		args[arguments++] = strdup(str.String());
		if (settingswindow->fixateCB->Value()==B_CONTROL_OFF) args[arguments++] = strdup("-nofix");
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-dummy");

		


	//	str.SetTo("mcn=");
	//	str << real_time_clock();
	//	args[arguments++] = strdup(str.String());
		//richtige reihenfolge und parameter für einzelne tracks
		for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			AudioRow *si=(AudioRow *)window1->view1->listview1->RowAt(i);
			str.SetTo("pregap=");
			str << (int)(si->GetPregap()*75);
			args[arguments++] = strdup(str.String());

			args[arguments++] = strdup("-swab");
			args[arguments++] = strdup("-audio");
			args[arguments++] = strdup("-pad");
			
//			str.SetTo("textfile=");
//			str.Append(GenerateInfFilename(i)->String());
//			args[arguments++] = strdup(str.String());
//			args[arguments++] = strdup("-text");
//			args[arguments++] = strdup("-useinfo");
			str.SetTo("tsize=");
//			temp.SetTo(base_folder).Append(si->Text());
//			file.SetTo(temp.String(),true);
//			MMediaInfo info=MMediaInfo(file);
//			filesize=info.Size();
//			// entweder es ist was schiefgelaufen, 
//			// oder es handelt sich um eine RAW-Datei...
//			if (filesize<0)
//				{
//				BFile f;
//				if ((f.SetTo(temp.String(),B_READ_ONLY)!=B_OK) ||
//					(f.GetSize(&filesize)!=B_OK))
//						// Fehler aufgetreten
//						{
//						ShowStatus(TFILEERROR);
//						sleep(2);
//						free(args);
//						HideStatus();
//						return;
//						}
//				f.Unset();
//				}
			str << si->GetBytes();
			args[arguments++] = strdup(str.String());
			if (strlen(si->GetCdrecordReadyList())>1) {
				str.SetTo("index=");
				str.Append(si->GetCdrecordReadyList());
				args[arguments++] = strdup(str.String());
			}
			
			args[arguments++] = strdup("-");
			}
		args[arguments]=NULL;
	
		printArguments("cdrecord non-streaming args",args);

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
			if (cdrecord_Thread<0)
				if (window1->Alert("Failure at: load_image(cdrecord)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); RemoveInfFiles();return;}
		SetStatus2(TWAITING,cdrecord_Thread);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);
//		working=true;
//		bf.working=&working;
//		bf.buffersize=4096000;
//		bf.in=toraw_out;
//		bf.out=cdrecord_in;
		
//		//Parameter für "toRAW" erstellen
		arguments=0;
		AudioRow *si;
		
//		toraw="";
//		toraw.Append(base_folder).append("cdtextfile.cdt");
//		CDTextFile *textfile=new CDTextFile(new BEntry(toraw.String()));
//		textfile->SetAlbum("");
		
		
		
		for (int i=0; i<window1->view1->listview1->iCountItems();i++)
			{
			si=(AudioRow *)window1->view1->listview1->RowAt(i);
			toraw.SetTo(base_folder);
			toraw.Append(si->GetFilename());
			if (!si->IsSplitted())
				ap.filename[arguments++] = strdup(toraw.String());
			}
		ap.trackcount=arguments;
		ap.fd=cdrecord_in;

		buffer_Thread=spawn_thread(AudioBufferProducer,"Buffer Thread",TORAW_PRIORITY,(void *)(&ap));
		rename_thread(buffer_Thread,BUFFERTHREADNAME);
		resume_thread(cdrecord_Thread);	

		arguments=0;
		while (find_thread(CDRECORDTHREADNAME)!=cdrecord_Thread) {
			arguments++;
			snooze(1000);
			if (arguments>10000) break;
		}
		printf("HELIOS: cdrecord_Thread=%ld\n", cdrecord_Thread);
		resume_thread(buffer_Thread);	


		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		int track=1;
		float length=0;	// in MB
		float op=0,progressed=0;
		char *info2;
		BString s;
		
		do {
			bytesread=read(cdrecord_out,(void *)buffer, 10000);
			if (bytesread>0)
				buffer[bytesread]=0;

			cdrecord_output->AddText(buffer);

			if (bytesread>0) {
				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					asprintf(&info,"%dx (%0.1f MB/s)",speed,176400.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				
				// Performing OPC...
				if (strstr(buffer,"Performing OPC...")) {
					SetStatus2(TPERFORMINGOPC,-1);
					SetPercentage(0.0);
				}

				// finished fixating...
				if (strstr(buffer,"Fixating time:")) {
					SetStatus2(TFINISHEDWRITING,-1);
				}

				// start burning...
				if (strstr(buffer,"Starting new track")) {
					SetStatus2(TBURNINGAUDIOCD,-1);
					op=0;progressed=0;
				}

				// Fixating...
				if (strstr(buffer,"Fixating...")) {
					SetStatus2(TFIXATING,-1);
					SetPercentage(100.0);
				}

				// get track number
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.RemoveFirst("Track");
					s.Remove(s.FindFirst(":"),s.Length()-s.FindFirst(":"));
					track=atoi(s.String());
					asprintf(&info2,"%s %d",TTRACK,track);
					if (track!=0)
					SetStatus2Info(window5->view5->progressSB->TrailingText(), info2);
				}		

				// get track length
				//if (length==0)
				if (strstr(buffer,"of")) {
					s=strstr(buffer,"of");
					s.RemoveFirst("of");
					s.Remove(s.FindFirst("MB"),s.Length()-s.FindFirst("MB"));
					length=atof(s.String());
				}		
	
				// get write position
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.Remove(0,9);
					s.Remove(s.FindFirst("of"),s.Length()-s.FindFirst("of"));
					progressed=atof(s.String());
					if (op<progressed) {
						op=progressed;
						if (length>0) {
							asprintf(&info,"%0.1f%%",100.0*(float)progressed/(float)length);
							asprintf(&info2,"%s %d",TTRACK,track);
							SetStatus2Info(info, info2);
							SetPercentage(100.0*(float)progressed/(float)length);
						}
					}
				}		
			}
			snooze(70000);
		} while (cdrecord_Thread==find_thread(CDRECORDTHREADNAME));
		RemoveInfFiles();
//		working=false;
//		bf.working=&working;
		snooze(100000);
		if (find_thread(BUFFERTHREADNAME)==buffer_Thread) kill_thread(buffer_Thread);
//		if (find_thread(TORAWTHREADNAME)==toraw_Thread) kill_thread(toraw_Thread);
//		close(toraw_in);close(toraw_out);close(toraw_err);
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);
		if (window5->interrupted) {HideStatus2(); free(args); return;}
			
		if (settingswindow->ejectwriterCB->Value()==B_CONTROL_ON)
			eject_media(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
		free(args);
		HideStatus2();

}


void Application1::BlankRW(char *mode)
{
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString endstr="";
		int in, out, err;
		char *info;
		// wait till a disc is in the drive and drive is ready
	
		window5->Lock();
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON)
			window5->SetTitle(TSTATUSWINDOWTITLESIMULATION); else
			window5->SetTitle(TSTATUSWINDOWTITLE);
		window5->Unlock();

		ShowStatus2(TWAITINGFORDISC,0);
		if (WaitForDisc(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())])==B_TIMEOUT) return;
		
		device << devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())];
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup("/boot/home/config/bin/cdrecord");
		args[arguments++] = strdup(device.String());
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-dummy");
		args[arguments++] = strdup(mode);
		args[arguments++] = strdup("-v");
		endstr << "-speed=" << (settingswindow->writerspeedSL->Value())*2;
		args[arguments++] = strdup(endstr.String());
		args[arguments]=NULL;
		
		printArguments("cdrecord non-streaming args",args);


		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,cdrecord_priority);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image()\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return;}
		SetStatus2(TWAITING,Thread);
		rename_thread(Thread,CDRECORDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		BString s;
		
		while (Thread==find_thread(CDRECORDTHREADNAME)) {
			bytesread=read(out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			
			
			cdrecord_output->AddText(buffer);
			
			
			if (bytesread>0) {

				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					asprintf(&info,"%dx (%0.1f MB/s)",speed,176400.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				

				// Fixating...
				if (strstr(buffer,"Blanking time:")) {
					SetStatus2(TFINISHEDERASING,-1);
					SetPercentage(100.0);
				} else
				// start blanking...
				if (strstr(buffer,"Blanking")) {
					SetStatus2(TBLANKINGCDRW,-1);
					SetPercentage(33.33);
				}


			}
			snooze(100000);
		}
		close(in);close(out);close(err);
		window5->Lock();
		window5->SetTitle(TSTATUSWINDOWTITLE);
		window5->Unlock();
		if (window5->interrupted) {HideStatus2(); free(args); return;}

		if (settingswindow->ejectwriterCB->Value()==B_CONTROL_ON)
			eject_media(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);

		HideStatus2();
		free(args);
}


// verschiebt einen Audio-eintrag um eine position nach oben
void Application1::MoveUp()
{
/*
has_project_changed=true;
int32 to, from;
//printf("%d\n",window1->view1->listview1->CountItems());
window1->Lock();
		for (int32 i=1; i<window1->view1->listview1->iCountItems();i++)
 			if (window1->view1->listview1->IsSelected(i)) {
 				to=i;
 				from=i;
  				to--;
				while ((((AudioRow *)(window1->view1->listview1->RowAt(to)))->IsSplitted()) ||
 					(((AudioRow *)(window1->view1->listview1->RowAt(to)))->GetSplitIndex()==0)) {
   					to--;
				};
			 	// move item when it is NOT a part of a splitted track
				if (!((((AudioRow *)(window1->view1->listview1->RowAt(from)))->IsSplitted()) ||
 					(((AudioRow *)(window1->view1->listview1->RowAt(from)))->GetSplitIndex()==0)))
					if (to>=0) window1->view1->listview1->MoveRow(from,to);
			}
window1->view1->listview1->Invalidate();
window1->Unlock();
*/
}

// verschiebt einen Audio-eintrag um eine position nach unten
void Application1::MoveDown()
{
/*
has_project_changed=true;
int32 max=window1->view1->listview1->CountItems();
int32 to;
window1->Lock();
		  for (int32 i=window1->view1->listview1->CountItems()-2;i>=0;i--)
 			if (window1->view1->listview1->IsItemSelected(i)) {
 				to=i;
  				to++;
				while ((((AudioItem *)(window1->view1->listview1->ItemAt(to)))->IsSplitted()) ||
 					(((AudioItem *)(window1->view1->listview1->ItemAt(to)))->IsParentSplitterTrack())) {
  				to++;
				};
			 	// move item when it is NOT a part of a splitted track
				if (!((((AudioItem *)(window1->view1->listview1->ItemAt(i)))->IsSplitted()) ||
 					(((AudioItem *)(window1->view1->listview1->ItemAt(i)))->IsParentSplitterTrack())))
					if (to<max) window1->view1->listview1->MoveItem(i,to);
			} 
window1->view1->listview1->Invalidate();
window1->Unlock();
*/
}





bool Application1::ReadImage(BString filename, bool alloweject) {

		int in, out, err;
		thread_id Thread;
		const char **args;
		BString str="f=";
		BString device="dev=";
		BString endstr="";
		int arguments=0;
		BEntry entry;
		char *info;
	
		// wait till a disc is in the drive and drive is ready
	
		ShowStatus2(TWAITINGFORDISC,0);
		if (WaitForDisc(devicepaths[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())])==B_TIMEOUT) return false;
		
		device << devices[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())];
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup("/boot/home/config/bin/readcd");
		args[arguments++] = strdup(device.String());
		str << filename.String();
		args[arguments++] = strdup(str.String());
//		args[arguments++] = strdup("-v");
		args[arguments]=NULL;

		printArguments("readcd args",args);

		// delete image file if it exists
		entry.SetTo(filename.String(), true);
		if (entry.Exists()) entry.Remove();
		entry.Unset();

		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,16);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image()\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return false;}
		SetStatus2(TREADINGCD,Thread);
		rename_thread(Thread,READCDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		float length=-1;
		float progressed, olddone=0;
		BString s;
		BStopWatch *watch=new BStopWatch("MBps",true);

		SetStatus2Info("--- MB/s");
		
		do {
			bytesread=read(err,(void *)buffer, 10000);
			buffer[bytesread]=0;
			if (bytesread>0) {
//				printf(">>>%s\n",buffer);
				if (length<0)
				if (strstr(buffer,"end:")) {
					s=strstr(buffer,"end:");
					s.RemoveFirst("end:");
					//s.Remove(s.FindFirst("Blocks"),s.Length());
					length=atof(s.String());
				} // strstr			

				if (strstr(buffer,"addr:")) {
					s=strstr(buffer,"addr:");
					s.Remove(0,5);
					s.Remove(s.FindFirst("cnt:"),s.Length()-s.FindFirst("cnt:"));
					progressed=atof(s.String());
					if (length>0) {
						if (progressed>olddone+3072) {
							asprintf(&info,"%0.1f MB/s",2048.0*(progressed-olddone)/(watch->ElapsedTime()*1.048576));
							olddone=progressed;
							SetStatus2Info(info);
							watch->Reset();
						}
						SetPercentage(100*progressed/length);
					}
				} // strstr			
			}
			snooze(10000);
		} while (Thread==find_thread(READCDTHREADNAME)) ;
		close(in);close(out);close(err);
		delete watch;
		if (window5->interrupted) {HideStatus2(); free(args); return false;}
		if (alloweject)
			if (settingswindow->ejectreaderCB->Value()==B_CONTROL_ON)
				eject_media(devicepaths[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())]);
			
		free(args);
		HideStatus2();
		return true;
}


bool Application1::MakeImageOnly(BString filename)
{
	bool ok=false;
	
	if (window1->view1->GetFilesystem()==2)
		ok=MakeBFSImage(filename.String()); else
		ok=MakeMKISOImage(filename.String());
	
	if (!ok) {
			BAlert *alert=new BAlert("error",TERROR,TOK);
			alert->Go();
		}
		
	return ok;
}

void Application1::eject_media
	(
	const char * dev
	)
{
	int 		fd;
	status_t	status;

	fd = open(dev, 0);
	// printf("ejecting %s\n", dev);
	status = ioctl(fd, B_EJECT_DEVICE, 0, 0);
	close(fd);
}


void Application1::CopyDataCD() {

	BString filename=settingswindow->heliospathTC->Text();
	int	timeout=DEFAULTTIMEOUT*10;
	char *info;

	if (filename[filename.Length()-1]!='/') filename.Append("/");
	filename.Append(DEFAULTIMAGEFILENAME);
	if (!ReadImage(filename)) return;
	// wait till door has been opened
	SetStatus2(TWAITING,0);
	if (settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())==settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked()))
	while (DeviceStatus(devicepaths[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())])!=E_DOOR_OPEN) {
		snooze(100000);
		timeout--;
		if ((timeout==0) || (window5->interrupted)){
			HideStatus2();
			return;
		}
		asprintf(&info,"%d\"",timeout/10);
		SetStatus2Info(info);
		SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*10));
		};

	BurnImage(filename);
	
}


void Application1::CopyOnTheFly()
{
		int arguments=0;
		thread_id buffer_Thread, readcd_Thread, cdrecord_Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString filesystem="";
		char *info;
		bool working;
		bufferfunc	bf;
		int readcd_in, readcd_out, readcd_err, cdrecord_in, cdrecord_out, cdrecord_err;

		// wait till a disc is in the drive and drive is ready
		if (WaitForDisc(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())])==B_TIMEOUT) return;
		if (WaitForDisc(devicepaths[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())])==B_TIMEOUT) return;
		

		//GetImageSize();
		device << devices[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())];
		args = (const char **)malloc(sizeof(char *) * (22));
		// Make READCD options
		args[arguments++] = strdup("/boot/home/config/bin/readcd");


		args[arguments++] = strdup(device.String());
		args[arguments++] = strdup("f=-");
		args[arguments]=NULL;

		printArguments("readcd streaming args",args);

		// launch mkisofs thread now
		readcd_Thread=pipe_command(arguments, args, readcd_in,readcd_out, readcd_err);
		set_thread_priority(readcd_Thread,READCD_PRIORITY-1);
			if (readcd_Thread<0)
				if (window1->Alert("Failure at: load_image(readcd)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return;}
		SetStatus2(TWAITING,readcd_Thread);
		rename_thread(readcd_Thread,READCDTHREADNAME);

		// Make CDRECORD options
		device="dev=";
		device << devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())];
		arguments=0;
		args[arguments++] = strdup("/boot/home/config/bin/cdrecord");
		args[arguments++] = strdup(device.String());
		if (settingswindow->burnproofCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("driveropts=burnproof");
		args[arguments++] = strdup("-v");
		if (settingswindow->allowoverburningCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-overburn");
		args[arguments++] = strdup("-waiti");
		args[arguments++] = strdup("-speed=");
		str.SetTo("");
		str << (settingswindow->writerspeedSL->Value())*2;
		args[arguments++] = strdup(str.String());
		if (settingswindow->fixateCB->Value()==B_CONTROL_OFF) args[arguments++] = strdup("-nofix");
		if (window1->view1->IsMultisession()) args[arguments++] = strdup("-multi");
		if (settingswindow->simulateCB->Value()==B_CONTROL_ON) args[arguments++] = strdup("-dummy");
		str.SetTo("");
		args[arguments++] = strdup("tsize=99999999");
		printf("HELIOS: IMAGESIZE IS %s\n",str.String());
		args[arguments++] = strdup("-data");
		args[arguments++] = strdup("-");
		args[arguments]=NULL;

		printArguments("cdrecord streaming args",args);

		// launch cdrecord thread now
		cdrecord_Thread=pipe_command(arguments, args, cdrecord_in, cdrecord_out, cdrecord_err);
		set_thread_priority(cdrecord_Thread,CDRECORD_PRIORITY);
			if (cdrecord_Thread<0)
				if (window1->Alert("Failure at: load_image(cdrecord)\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return;}
		SetStatus2(TWAITING,cdrecord_Thread);
		rename_thread(cdrecord_Thread,CDRECORDTHREADNAME);
		working=true;
		bf.working=&working;
		bf.buffersize=4*1024;
		bf.in=readcd_out;
		bf.out=cdrecord_in;

		buffer_Thread=spawn_thread(DataBuffer,"Buffer Thread",TORAW_PRIORITY,(void *)(&bf));
		rename_thread(buffer_Thread,BUFFERTHREADNAME);
		resume_thread(buffer_Thread);	
		resume_thread(readcd_Thread);	
		resume_thread(cdrecord_Thread);	





		size_t bytesread=0;
		char buffer[10000];
		int speed=0;
		int track=1;
		float length=0;	// in MB
		float op=0,progressed=0;
		char *info2;
		BString s;
		
		do {
			bytesread=read(cdrecord_out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			//printf("HELIOSBUFFER: >%s<\n",buffer);
			if (bytesread>0) {
				// get speed
				if (speed==0)
				if (strstr(buffer,"at speed")) {
					s=strstr(buffer,"at speed");
					s.Remove(0,8);
					s.Remove(s.FindFirst("in"),s.Length()-s.FindFirst("in"));
					speed=atoi(s.String());
					asprintf(&info,"%dx (%0.1f MB/s)",speed,176400.0*(float)speed/(1024.0*1024.0));
					SetStatus2Info(info);
				}
				
				// Performing OPC...
				if (strstr(buffer,"Performing OPC...")) {
					SetStatus2(TPERFORMINGOPC,-1);
				}

				// finished fixating...
				if (strstr(buffer,"Fixating time:")) {
					SetStatus2(TFINISHEDWRITING,-1);
				}

				// start burning...
				if (strstr(buffer,"Starting new track")) {
					SetStatus2(TBURNINGDATACD,-1);
				}

				// Fixating...
				if (strstr(buffer,"Fixating...")) {
					SetStatus2(TFIXATING,-1);
				}

				// get track number
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.RemoveFirst("Track");
					s.Remove(s.FindFirst(":"),s.Length()-s.FindFirst(":"));
					track=atoi(s.String());
				}		

				// get track length
				if (length==0)
				if (strstr(buffer,"of")) {
					s=strstr(buffer,"of");
					s.RemoveFirst("of");
					s.Remove(s.FindFirst("MB"),s.Length()-s.FindFirst("MB"));
					length=atof(s.String());
				}		
	
				// get write position
				if (strstr(buffer,"Track")) {
					s=strstr(buffer,"Track");
					s.Remove(0,9);
					s.Remove(s.FindFirst("of"),s.Length()-s.FindFirst("of"));
					progressed=atof(s.String());
					if (op<progressed) {
						op=progressed;
						if (length>0) {
							asprintf(&info,"%0.1f%%",100.0*(float)progressed/(float)length);
							asprintf(&info2,"%s %d",TTRACK,track);
							SetStatus2Info(info, info2);
							SetPercentage(100.0*(float)progressed/(float)length);
						}
					}
				}		
			}
			snooze(200000);
		}while (cdrecord_Thread==find_thread(CDRECORDTHREADNAME));
		working=false;
		bf.working=&working;snooze(100000);
		if (find_thread(BUFFERTHREADNAME)==buffer_Thread) kill_thread(buffer_Thread);
		if (find_thread(READCDTHREADNAME)==readcd_Thread) kill_thread(readcd_Thread);
		close(readcd_in);close(readcd_out);close(readcd_err);
		close(cdrecord_in);close(cdrecord_out);close(cdrecord_err);
		if (window5->interrupted) {HideStatus2(); free(args); return;}
			
		if (settingswindow->ejectwriterCB->Value()==B_CONTROL_ON)
			eject_media(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())]);
		if (settingswindow->ejectreaderCB->Value()==B_CONTROL_ON)
			eject_media(devicepaths[settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked())]);
		free(args);
		HideStatus2();
}


int Application1::MountBFSImage(const char *filename, const char *mountpoint) {
		int	timeout=DEFAULTTIMEOUT/5;
		char *info;
		int retvalue;
	SetStatus2(TMOUNTING,0);
	while ((retvalue=mount("bfs",mountpoint,filename,2,NULL,0))<0) {
		snooze(5000000);
		timeout--;
		if ((timeout==0) || (window5->interrupted)){
			return -1;
		}
		asprintf(&info,"%s",TWAITING);
		SetStatus2Info(info);
		SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT/5));
	};
	return retvalue;
}

int Application1::UnmountBFSImage(const char *mountpoint) {
		int	timeout=DEFAULTTIMEOUT;
		char *info;
		int retvalue;
		BString s;

	SetStatus2(TUNMOUNTING,0);
	while ((retvalue=unmount(mountpoint))<0) {
		snooze(1000000);
		timeout--;
		if ((timeout==0) || (window5->interrupted)){
			return -1;
		}
		asprintf(&info,"%d\"",timeout);
		SetStatus2Info(info);
		SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT));
	};
	s.SetTo("/boot/beos/bin/rmdir ").Append(mountpoint);
	system(s.String());
	return retvalue;
}



// Returns size of free space of a volume by passing the path of the device only.
size_t FreeVolumeSpace(const char *path) {
	
	fs_info	info;
	dev_t	device;
	
	
	device=dev_for_path(path);
	if (device>=0) {
		if (fs_stat_dev(device, &info)==0)
			return info.free_blocks*info.block_size;
	}
	return 0;	
}




bool Application1::MakeBFSImage(const char *filename) {

		BFile file;
		BDirectory dir;
		BEntry entry;
		const char **args;
		status_t ret;
		int arguments=0;
		thread_id Thread;
		BString str="";
		size_t		FATsize,size, sizetowrite;
		int buffersize=1048576;
		status_t returnvalue=B_OK;
		extern char **environ;
		int	timeout=DEFAULTTIMEOUT;
		char *info;
		char *buffer(NULL);



				
		
		imgsize=0;
		files=0;
		directories=0;
		
		// delete image file if it exists
		entry.SetTo(filename);
		if (entry.Exists()) entry.Remove();
		entry.Unset();
		
		// get size of directories and files
		GetSize(BDirectory(base_folder.String()));
		/*
		// displays info about files and folders
		str="";
		str<<imgsize<<" bytes\n" <<files<<" files\n"<<directories<< " folders";
		window1->Alert((char *)str.String());
		*/
		
		args = (const char **)malloc(sizeof(char *) * (22));
		str.SetTo("mkdir ").Append(IMG_MOUNTPOINT);
		printf("HELIOS: creating mount point\n");
		system(str.String());
		imgsize+=files*4096+directories*133200;	// die informationen für fensterpositionen und
								// positionen der icons in den fenstern
								// werden auch abgespeichert...
		FATsize=(5144*1024)+64*imgsize/(1024*1024)+384;
		imgsize+=FATsize;
		file.SetTo(filename,B_READ_WRITE | B_CREATE_FILE);
		ret=file.SetSize(imgsize);
		// shit happens...
		if (ret==B_DEVICE_FULL)
			{
			BAlert *alert=new BAlert("device full",TNOSPACE,TOK);
			alert->Go();
			free(args);
			return false;
			}
		size=0;
		printf("SIZE: imagesize is %lld bytes\n",imgsize);
		sizetowrite=buffersize;

		size_t olddone=0;
		
		ShowStatus2(TCREATINGIMAGEFILE,0);
		SetStatus2Info("--- MB/s");
		BStopWatch *watch=new BStopWatch("MBps",true);

		buffer=(char *)malloc(buffersize);
		for (int i=0; i<buffersize; i++) buffer[i]=0x0;		

		while (sizetowrite>0) {
			if (window5->interrupted){
				delete watch;
				free(args);
				free(buffer);
				file.Unset();
				HideStatus2();
				return false;
			}
			if ((imgsize-size)>buffersize) sizetowrite=buffersize; else sizetowrite=imgsize-size;
		 	size+=file.Write((const void *)buffer,sizetowrite);


						if (size>olddone+5*buffersize) {
							asprintf(&info,"%0.1f MB/s",((float)size-(float)olddone)/(watch->ElapsedTime()*0.953674316));
							olddone=size;
							SetStatus2Info(info);
							watch->Reset();
						}
		 	//asprintf(&info,"&0.1f%% (xx.x MB/s)",(float)size*100.0/((float)imgsize));
			SetPercentage((float)size*100/((float)imgsize));
		};
		file.Unset();
		delete watch;
		free(buffer);
		// create a Be File System
		// initialize the image file as a BFS image
		arguments=0;
		args[arguments++] = strdup("/boot/beos/bin/mkbfs");
		args[arguments++] = strdup("2048");
		args[arguments++] = strdup(filename);
		args[arguments++] = strdup(window1->view1->GetVolumeName());
		args[arguments]=NULL;

		Thread=load_image(arguments, args, (const char **)environ);
		set_thread_priority(Thread,20);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image()\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args);return false;}
		SetStatus2(TCREATINGIMAGEFILE,Thread);
		printf("HELIOS: creating Be File System\n");
		wait_for_thread(Thread,&returnvalue);
			if (window5->interrupted) {HideStatus2(); free(args);return false;}
		sleep(4);
		// mount the image file!
		printf("HELIOS: mounting image file\n");
		if (MountBFSImage(filename, IMG_MOUNTPOINT)<0) {UnmountBFSImage(IMG_MOUNTPOINT);HideStatus2(); free(args);return false;}

//		printf("HELIOS: Checking for home directory\n");
//		timeout=DEFAULTTIMEOUT*2;
//		SetStatus2(TWAITING,0);
//		str.SetTo(IMG_MOUNTPOINT).Append("/home");
//		entry.SetTo(str.String());
//		while (!entry.Exists()) {
//			snooze(500000);
//			timeout--;
//			if ((timeout==0) || (window5->interrupted)){
//				free(args);
//				entry.Unset();
//				UnmountBFSImage(IMG_MOUNTPOINT);
//				HideStatus2();
//				return false;
//			}
//			asprintf(&info,"%d\"",timeout*100/(DEFAULTTIMEOUT*2));
//			SetStatus2Info(info);
//			SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*2));
//		};
//		entry.Unset();
		snooze(5000000);
		// sometimes, freshly created image files aren't empty...
		// so, delete all files, please.
		printf("HELIOS: erasing files in mounted image file\n");
		DeleteAllIn(BDirectory(IMG_MOUNTPOINT));
		snooze(5000000);
//		str.SetTo("/boot/beos/bin/rm -r -v ").Append(IMG_MOUNTPOINT).Append("/*");
//		system(str.String());



//		printf("HELIOS: Checking for empty space in image file.\n");
//		timeout=DEFAULTTIMEOUT*2;
//		SetStatus2(TWAITING,0);
//		while ((size=FreeVolumeSpace(IMG_MOUNTPOINT))<imgsize-FATsize) {
//			snooze(500000);
//			timeout--;
//			if ((timeout==0) || (window5->interrupted)){
//				free(args);
//				UnmountBFSImage(IMG_MOUNTPOINT);
//				HideStatus2();
//				return false;
//			}
//			asprintf(&info,"%0.1f%%",100*size/(imgsize-FATsize));
//			SetStatus2Info(TWAITING);
//			SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*2));
//		};



		printf("HELIOS: copying files to image file.\n");
		// copy files into mounted image file
		arguments=0;
		args[arguments++] = strdup("/boot/beos/bin/cp");
		args[arguments++] = strdup("-r");
		str.SetTo(base_folder.String()).Append("");
		args[arguments++] = strdup(str.String());
		str.SetTo(IMG_MOUNTPOINT).Append("/");
		args[arguments++] = strdup(str.String());
		args[arguments]=NULL;

		Thread=load_image(arguments, args, (const char **)environ);
		set_thread_priority(Thread,12);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image()\n\nUnknown load error. Sorry.")==true) 
						 {UnmountBFSImage(IMG_MOUNTPOINT);HideStatus2(); free(args);return false;}
		SetStatus2(TCOPYINGFILES,Thread);

		rename_thread(Thread,CPTHREADNAME);
		resume_thread(Thread);
		timeout=DEFAULTTIMEOUT*100;
		
		// waiting for copy-thread
		while (find_thread(CPTHREADNAME)!=Thread) {
			snooze(10000);
			timeout--;
			if ((timeout==0) || (window5->interrupted)){
				free(args);
				UnmountBFSImage(IMG_MOUNTPOINT);
				HideStatus2();
				return false;
			}
			SetPercentage((float)timeout*100/((float)DEFAULTTIMEOUT*100));
		};

		// COPYING FILES
		while (find_thread(CPTHREADNAME)==Thread) {
			sleep(2);
			if (window5->interrupted){
				free(args);
				UnmountBFSImage(IMG_MOUNTPOINT);
				HideStatus2();
				return false;
			}
//			asprintf(&info,"%0.1f%%",100*(size=FreeVolumeSpace(IMG_MOUNTPOINT))/(imgsize-FATsize));
//			SetStatus2Info(info);
//			SetPercentage((float)100*size/(imgsize-FATsize));
		};



		// unmount image file
		if (UnmountBFSImage(IMG_MOUNTPOINT)<0) {HideStatus2(); free(args);return false;}
			
		dir.Unset();
		HideStatus2();
		free(args);
		return true;
}



int Application1::MakeMKISOFSCommandline(const char **args, const char *filename, bool printsize) {
		int arguments=0;

		args[arguments++] = strdup("/boot/home/config/bin/mkisofs");
		args[arguments++] = strdup("-gui");
		args[arguments++] = strdup("-follow-links");
		if (printsize)
			args[arguments++] = strdup("-print-size");
		args[arguments++] = strdup("-R");
		args[arguments++] = strdup(window1->view1->datapanel->GetFileSystem());
			
		if (strlen(settingswindow->applicationTC->Text())!=0) {
			args[arguments++] = strdup("-A");
			args[arguments++] = strdup(settingswindow->applicationTC->Text());
		}

		if (strlen(settingswindow->preparerTC->Text())!=0) {
			args[arguments++] = strdup("-p");
			args[arguments++] = strdup(settingswindow->preparerTC->Text());
		}
		
		if (strlen(settingswindow->publisherTC->Text())!=0) {
			args[arguments++] = strdup("-P");
			args[arguments++] = strdup(settingswindow->publisherTC->Text());
		}

		// bootable CD
		if (window1->view1->CDType()==BOOTABLECD_INDEX) {
			BString	bootimage=window1->view1->GetImageNameBootCD();
			bootimage.RemoveFirst("/");
			// add where to find the boot image file
			args[arguments++] = strdup("-b");
			args[arguments++] = strdup(bootimage.String());
			// and where the boot catalog can be found
			
			// if there is a dot in the image file name, cut off the
			// string after the dot and add "catalog"
			// the dot has to be after the last slash
			if (bootimage.FindLast(".")>bootimage.FindLast("/")) {
				bootimage.Remove(bootimage.FindLast("."), bootimage.Length()-bootimage.FindLast("."));
			}
				bootimage.Append(".cat");
			
			args[arguments++] = strdup("-c");
			args[arguments++] = strdup(bootimage.String());
		}
	

		args[arguments++] = strdup("-V");
		args[arguments++] = strdup(window1->view1->GetVolumeName());
		
		// output to file else to stdout...
		if ((strlen(filename)!=0) && (!printsize)) {
			args[arguments++] = strdup("-o");
			args[arguments++] = strdup(filename);
		}	// else output to stdout
		
		args[arguments++] = strdup(base_folder.String());
		args[arguments]=NULL;
		
		// send arguments to stdout
		if (!printsize)
			printArguments("MKISOFS Arguments",args);
		
		return arguments;
}



bool Application1::MakeMKISOImage(const char *filename) {

		const char **args;
		int arguments=0;
		thread_id Thread;
		BString str="";
		int in, out, err;
				
		// allocate some memory for the command line array
		args = (const char **)malloc(sizeof(char *) * (23));
		// fill array with arguments
		arguments=MakeMKISOFSCommandline(args, filename);
		// and _please_ launch a new thread!
		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,16);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image()\n\nUnknown load error. Sorry.")==true) 
						 {HideStatus2(); free(args); return false;}
		ShowStatus2(TCREATINGIMAGEFILE,Thread);
		rename_thread(Thread,MKISOFSTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		char prev;
		char *info;
		int32 length;
		float percentage;
		
		while (Thread==find_thread(MKISOFSTHREADNAME)) {
			bytesread=read(err,(void *)buffer, 10000);
			buffer[bytesread]=0;
			if (bytesread>0) {
				if (strstr(buffer," done, estimate")) {
					buffer[(length=(bytesread-strlen(strstr(buffer," done, estimate"))-1))]=0;
					if (buffer[length-1]==' ') {
						prev=buffer[length-2];	
						buffer[length-2]='0';
						buffer[length-1]=prev;
					}
					percentage=atof(buffer);
					asprintf(&info,"%0.1f%%",percentage);
					SetStatus2Info(info);
					SetPercentage(percentage);
				} // strstr			
			}
			snooze(70000);
		}
		close(in);close(out);close(err);
		if (window5->interrupted) {HideStatus2(); free(args); return false;}
			
		free(args);
		HideStatus2();
		return true;
}


void Application1::GetSize(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
//	static off_t size=0;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, true);
	//	printf("SIZE: %lld\n",imgsize);
		if (entry.IsDirectory())
			{
			directories++;
	//		printf("DIRECTORY\n");
			GetSize(BDirectory(&entry));
			//entry->GetParent(&dir);
			}
			else
			{
			files++;
	//		printf("FILE\n");
			BFile file;
			char buf[B_ATTR_NAME_LENGTH];
			off_t s=0;
			attr_info info;
			
			file.SetTo(&entry, B_READ_ONLY);
			while (file.GetNextAttrName(buf)==B_OK)
				{
				file.GetAttrInfo((const char *)buf,&info);
				imgsize+=info.size;
				}
			file.GetSize(&s);
			file.Unset();
			if (s % 2048!=0) imgsize+=(int(s / 2048)+1)*2048; else
			imgsize+=s;
			}	
			entry.Unset();		
	}
	
}


void
Application1::SaveSettings() {

	BFile conf;
	
	settings._burnproof=(settingswindow->burnproofCB->Value()==B_CONTROL_ON);
	settings._simulation=(settingswindow->simulateCB->Value()==B_CONTROL_ON);
	settings._fixate=(settingswindow->fixateCB->Value()==B_CONTROL_ON);
	settings._normalize=window1->view1->IsMultisession();
	settings._streaming=(settingswindow->streamingCB->Value()==B_CONTROL_ON);
	settings._ejectwriter=(settingswindow->ejectwriterCB->Value()==B_CONTROL_ON);
	settings._ejectreader=(settingswindow->ejectreaderCB->Value()==B_CONTROL_ON);
	settings._writeDAO=(settingswindow->writeDAOCB->Value()==B_CONTROL_ON);
	settings._allowoverburning=(settingswindow->allowoverburningCB->Value()==B_CONTROL_ON);
	sprintf(settings._volumename,"%s", window1->view1->GetVolumeName());
	sprintf(settings._volumenameBootCD,"%s", window1->view1->GetVolumeNameBootCD());
	sprintf(settings._publisher,"%s", settingswindow->publisherTC->Text());
	sprintf(settings._preparer,"%s", settingswindow->preparerTC->Text());
	sprintf(settings._application,"%s", settingswindow->applicationTC->Text());
	sprintf(settings._heliospath,"%s", settingswindow->heliospathTC->Text());
	settings._recorderspeed=settingswindow->writerspeedSL->Value()*2;
	settings._recorderdevice=settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked());
	settings._readerdevice=settingswindow->readerPUM->IndexOf(settingswindow->readerPUM->FindMarked());
	settings._filesystem=window1->view1->GetFilesystem();
	settings._filesystemBootCD=window1->view1->GetFilesystemBootCD();
	sprintf(settings._eltoritoimage,"%s",window1->view1->GetImageNameBootCD());
	sprintf(settings._imagename,"%s",window4->view4->textcontrol4->Text());
	sprintf(settings._cdtextalbum,"%s",window1->view1->GetCDAlbum());
	settings._onthefly=(settingswindow->twodrivecopyCB->Value()==B_CONTROL_ON);
	settings._writeCDText=window1->view1->WriteCDText();
	settings._protectAudioCD=window1->view1->ProtectAudioCD();
	
	// get the window positions
	settings._window1position=window1->Frame();
	settings._window2position=window2->Frame();
	settings._window4position=window4->Frame();
	settings._settingswindowposition=settingswindow->Frame();
	settings._cdrecordlogwindow=cdrecord_output->Frame();
	settings._mkisofslogwindow=mkisofs_output->Frame();
	
	if (window1->view1->CDType()==DATACD_INDEX) settings._cdtype=0; else settings._cdtype=1;
	if (conf.SetTo(SETTINGS_FILE,B_READ_WRITE | B_CREATE_FILE)==B_OK)
		{
		conf.Write(&settings,sizeof(settings_struct));
		conf.Unset();		
		}
}



void
Application1::ReadSettings() {

	BFile conf;
	
		settings._burnproof=false;
		settings._simulation=false;
		settings._fixate=true;
		settings._normalize=false;
		settings._streaming=false;
		settings._ejectreader=true;
		settings._ejectwriter=true;
		settings._writeDAO=true;
		settings._recorderspeed=8;
		settings._recorderdevice=0;
		settings._filesystem=5;			//windows joliet
		settings._filesystemBootCD=5;			//windows joliet
		sprintf(settings._volumename,"untitled");
		sprintf(settings._volumenameBootCD,"untitled");
		sprintf(settings._publisher,"BeOS");
		sprintf(settings._preparer,"BeOS");
		sprintf(settings._application,TWINDOWTITLE); 	// sth like "Helios CDRECORDER x.y"
		sprintf(settings._heliospath,"/boot/Helios/");
		sprintf(settings._lastprojectfile, "");
		settings._cdtype=0;
		sprintf(settings._imagename,"/boot/home/image.img");
		settings._onthefly=false;
		settings._readerdevice=0;
		// set standard window positions
		settings._window1position=window1->Frame();
		settings._window2position=window2->Frame();
		settings._window4position=window4->Frame();
		settings._settingswindowposition=settingswindow->Frame();
		settings._cdrecordlogwindow=BRect(150,150, 400, 480);
		settings._mkisofslogwindow=BRect(150,150, 400, 480);
		sprintf(settings._s_publisher,"%s",settings._publisher);
		sprintf(settings._s_preparer,"%s",settings._preparer);
		sprintf(settings._s_application,"%s",settings._application);
		sprintf(settings._s_volumename,"%s",settings._volumename);
		settings._s_cdtype=settings._cdtype;
		settings._s_filesystem=settings._filesystem;
		sprintf(settings._s_currentpath,"%s",settings._heliospath);
		sprintf(settings._s_basepath,"%s",settings._heliospath);
		sprintf(settings._recentprojects[0],"");
		sprintf(settings._recentprojects[1],"");
		sprintf(settings._recentprojects[2],"");
		sprintf(settings._recentprojects[3],"");
		sprintf(settings._recentprojects[4],"");
		sprintf(settings._cdtextalbum,"");
		settings._allowoverburning=false;
		settings._writeCDText=false;
		settings._protectAudioCD=false;
		sprintf(settings._eltoritoimage,"");

	bool ok=false;
	off_t filesize;
	
	settingswindow->Lock();
	window1->Lock();
	window4->Lock();
	ok=conf.SetTo(SETTINGS_FILE,B_READ_WRITE)==B_OK;
	if (ok)
		{
		conf.GetSize(&filesize);
		ok=(filesize==sizeof(settings));
		conf.Unset();
		}
	if (!ok)
		// file not found
		{
		conf.SetTo(SETTINGS_FILE,B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
		conf.Write(&settings,sizeof(settings));
		conf.Unset();		
		} else
	if (conf.SetTo(SETTINGS_FILE,B_READ_WRITE)==B_OK)
		{
		conf.Read(&settings,sizeof(settings));
		conf.Unset();		
		}		

		if (settings._burnproof) 	settingswindow->burnproofCB->SetValue(B_CONTROL_ON); else
						settingswindow->burnproofCB->SetValue(B_CONTROL_OFF);
		if (settings._simulation) 	settingswindow->simulateCB->SetValue(B_CONTROL_ON); else
						settingswindow->simulateCB->SetValue(B_CONTROL_OFF);
		if (settings._fixate) 		settingswindow->fixateCB->SetValue(B_CONTROL_ON); else
						settingswindow->fixateCB->SetValue(B_CONTROL_OFF);
		if (settings._streaming) 	settingswindow->streamingCB->SetValue(B_CONTROL_ON); else
						settingswindow->streamingCB->SetValue(B_CONTROL_OFF);
		if (settings._ejectwriter) 	settingswindow->ejectwriterCB->SetValue(B_CONTROL_ON); else
						settingswindow->ejectwriterCB->SetValue(B_CONTROL_OFF);
		if (settings._ejectreader) 	settingswindow->ejectreaderCB->SetValue(B_CONTROL_ON); else
						settingswindow->ejectreaderCB->SetValue(B_CONTROL_OFF);
		if (settings._writeDAO) 	settingswindow->writeDAOCB->SetValue(B_CONTROL_ON); else
						settingswindow->writeDAOCB->SetValue(B_CONTROL_OFF);
		window1->view1->SetMultisession(settings._normalize);
		if (settings._onthefly) 	settingswindow->twodrivecopyCB->SetValue(B_CONTROL_ON); else
						settingswindow->twodrivecopyCB->SetValue(B_CONTROL_OFF);
		if (settings._allowoverburning) 	settingswindow->allowoverburningCB->SetValue(B_CONTROL_ON); else
						settingswindow->allowoverburningCB->SetValue(B_CONTROL_OFF);

		// set saved window positions
		window1->MoveTo(settings._window1position.LeftTop());
		window2->MoveTo(settings._window2position.LeftTop());
		window4->MoveTo(settings._window4position.LeftTop());
		settingswindow->MoveTo(settings._settingswindowposition.LeftTop());
		cdrecord_output->MoveTo(settings._cdrecordlogwindow.LeftTop());
		mkisofs_output->MoveTo(settings._mkisofslogwindow.LeftTop());
	
		window1->ResizeTo(settings._window1position.Width(), settings._window1position.Height());
		window2->ResizeTo(settings._window2position.Width(), settings._window2position.Height());
		window4->ResizeTo(settings._window4position.Width(), settings._window4position.Height());
		settingswindow->ResizeTo(settings._settingswindowposition.Width(), settings._settingswindowposition.Height());
		cdrecord_output->ResizeTo(settings._cdrecordlogwindow.Width(), settings._cdrecordlogwindow.Height());
		mkisofs_output->ResizeTo(settings._mkisofslogwindow.Width(), settings._mkisofslogwindow.Height());
		
		window1->view1->SetProtection(settings._protectAudioCD);
		window1->view1->SetCDText(settings._writeCDText);
		window1->view1->SetCDAlbum(settings._cdtextalbum);
		window1->view1->SetImageNameBootCD(settings._eltoritoimage);

		window4->view4->textcontrol4->SetText(settings._imagename);
		window1->view1->SetVolumeName(settings._volumename);
		window1->view1->SetVolumeNameBootCD(settings._volumenameBootCD);
		settingswindow->publisherTC->SetText(settings._publisher);
		settingswindow->preparerTC->SetText(settings._preparer);
		settingswindow->applicationTC->SetText(settings._application);
		settingswindow->heliospathTC->SetText(settings._heliospath);
		settingswindow->writerspeedSL->SetValue(settings._recorderspeed/2);
		
		while (settings._recorderdevice>=settingswindow->writerPUM->CountItems()) settings._recorderdevice--;
		settingswindow->writerPUM->ItemAt(settings._recorderdevice)->SetMarked(true);
		while (settings._readerdevice>=settingswindow->readerPUM->CountItems()) settings._readerdevice--;
		settingswindow->readerPUM->ItemAt(settings._readerdevice)->SetMarked(true);
		
		window1->view1->SetFilesystem(settings._filesystem);
		window1->view1->SetFilesystemBootCD(settings._filesystemBootCD);
	//	if (window1->view1->CDTypeM->CountItems()>settings._cdtype)
			window1->view1->CDTypeM->ItemAt(settings._cdtype)->SetMarked(true);
		BString str=TRECORDERSPEED;
		str << " (" << settingswindow->writerspeedSL->Value()*2 << "x)";
		settingswindow->writerspeedSV->SetText(str.String());
		window1->Unlock();
		window4->Unlock();
		settingswindow->Unlock();
		
		BDirectory HeliosDir;
		BString bf=settings._heliospath;
		bf.RemoveLast("/");
		if (HeliosDir.SetTo(settings._heliospath)!=B_OK)
			HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
		HeliosDir.Unset();
		
		ShowRecentProjects();
		if (strlen(settings._lastprojectfile)!=0)
		LoadProject(new BEntry(settings._lastprojectfile, true)); else
		be_app->PostMessage(NEWPROJECT_MIMSG);
}


void
Application1::Calculate_time()
{
off_t size2=window1->size;
if (settingswindow->fixateCB->Value()==B_CONTROL_ON) size2+=13824000;
uint64 r=(2352*(uint64)size2/(2048*176400*(uint64)settingswindow->writerspeedSL->Value()*2));
BString str="";
r+=10; // 10 Sekunden Wartezeit von cdrecord
r+=10; // und 10 sekunden wartezeit für spinup/spindown
r+=10; // und dann nochmal 10 sekunden, für alle fälle...
uint64 u=r-60*(uint32)(r/60);
char *s="";

if (u<10) s="0"; else s="";
str << (off_t)(r/60) << "m " << s << (off_t)u << "s";
window1->Lock();
window1->view1->box2->stringview6->SetText(str.String());
window1->Unlock();
}


void Application1::GetFolder(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, true);
		if (entry.IsDirectory())
			{
			directories++;
			GetFolder(BDirectory(&entry));
			}
			else
			files++;
	}
}


void Application1::UpdateTrackNumbers() {

//	RemoveFileTrackNumbers();
	window1->Lock();
	if (window1->view1->CDType()==AUDIOCD_INDEX)
		for (int i=0; i<window1->view1->listview1->iCountItems();i++) {
			((AudioRow *)(window1->view1->listview1->RowAt(i)))->SetTrackNumber(i+1);
//			window1->view1->listview1->InvalidateItem(i);
		}
	window1->Unlock();
//	AddFileTrackNumbers();
}


int Application1::DeviceStatus(const char *path) {

	if ( path == NULL || strlen(path) == 0 )
		return E_ERROR;
		
	// okay, path point to a device
	int 		fd;
	status_t	status;
	status_t	media_status;

	fd = open(path, 0);
	if (fd < 0 )
		return E_ERROR;
		
	status = ioctl(fd, B_GET_MEDIA_STATUS, &media_status, sizeof(media_status));
	close(fd);
	
	if ( status != B_OK )
		return E_ERROR;
	
	switch(media_status)
		{
		case B_NO_ERROR:
			return E_NO_ERROR;
		case B_DEV_NO_MEDIA:
			return E_NO_DISC;
		case B_DEV_NOT_READY:
			return E_NOT_READY;
		case B_DEV_MEDIA_CHANGED:
			return E_MEDIA_CHANGED;
		case B_DEV_DOOR_OPEN:
			return E_DOOR_OPEN;
		default:
			return E_ERROR;
		};
}






void Application1::DeleteAllIn(BDirectory dir) {

	int32 c=dir.CountEntries();
	BEntry entry;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, false);
		if (entry.IsDirectory()) {
			DeleteAllIn(BDirectory(&entry));
			entry.Remove();
		}
		else
		entry.Remove();
	}
}


void Application1::ShowOpenPanel() {
	
	BEntry		*entry=new BEntry(settings._heliospath, true);
	entry_ref	ref;
	
	entry->GetRef(&ref);
	if (filepanel!=NULL) delete filepanel;
	filepanel=new BFilePanel(B_OPEN_PANEL, &be_app_messenger, new entry_ref(ref),
				B_FILE_NODE, false, new BMessage(OPEN_FPMSG));
	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, C_OPEN);
	filepanel->SetButtonLabel(B_CANCEL_BUTTON, TCANCEL);
	filepanel->Show();

	entry->Unset();
	delete entry;
}


void Application1::ShowFolderPanel() {
	
	BEntry		*entry=new BEntry(settings._heliospath, true);
	entry_ref	ref;
	
	entry->GetRef(&ref);
	if (filepanel!=NULL) delete filepanel;
	filepanel=new BFilePanel(B_OPEN_PANEL, &be_app_messenger, new entry_ref(ref),
				B_DIRECTORY_NODE, false, new BMessage(FOLDER_FPMSG));
	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, C_SELECT);
	filepanel->SetButtonLabel(B_CANCEL_BUTTON, TCANCEL);
	
	// display some helpful text in the background view of the file panel,
	// for example for what reason this panel popped up.
	filepanel->Window()->Lock();
	BView *background = filepanel->Window()->ChildAt(0);
	BView *files = background->FindView("PoseView");
	BView *cancel_button = background->FindView("cancel button");
	//background->LockLooper();
	BStringView	*filepanelSV=new BStringView(BRect(files->Frame().left-2, cancel_button->Frame().top, cancel_button->Frame().left-8, cancel_button->Frame().bottom-1), "infoSV", TFOLDERPANELHELP, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	background->AddChild(filepanelSV);
	//background->UnlockLooper();
	filepanel->Window()->Unlock();
	filepanel->Show();

	entry->Unset();
	delete entry;
}


void Application1::AddToRecent(BString *filename) {

	// check if there is already a project with the same name
	for (int i=0; i<5; i++)
		if (strcmp(settings._recentprojects[i], filename->String())==0) return;

	// shift projects in array
	for (int i=0; i<4; i++) {
		sprintf(settings._recentprojects[i], "%s", settings._recentprojects[i+1]);
	}
	sprintf(settings._recentprojects[4], "%s", filename->String());
}


void Application1::ShowRecentProjects() {

	BEntry	entry;
	char	*filename;
	
	filename=(char *)malloc(B_FILE_NAME_LENGTH);
	window1->Lock();
	while (window1->view1->openrecentSM->CountItems()>0) window1->view1->openrecentSM->RemoveItem(0L);
	for (int i=0; i<5; i++) {
		if (strlen(settings._recentprojects[i])>0) {
			// add if project exists, only
			if (entry.SetTo(settings._recentprojects[i], true)==B_OK) {
					entry.GetName(filename);
					BMessage *msg=new BMessage(OPEN_FPMSG);		

					entry_ref	*ref=new entry_ref();
					entry.GetRef(ref);
					
					msg->AddRef("refs", ref);
					BMenuItem	*mi=new BMenuItem(filename, msg);
					mi->SetTarget(be_app_messenger);
					if (!entry.Exists()) mi->SetEnabled(false);
					window1->view1->openrecentSM->AddItem(mi, 0);
			}
		}
	}
	window1->Unlock();
	free(filename);
}


void Application1::ShowSavePanel() {
	
	BEntry		*entry=new BEntry(settings._heliospath, true);
	entry_ref	ref;
	
	entry->GetRef(&ref);
	if (filepanel!=NULL) delete filepanel;
	filepanel=new BFilePanel(B_SAVE_PANEL, &be_app_messenger, new entry_ref(ref),
				0, false, new BMessage(SAVE_FPMSG));
	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, C_SAVE);
	filepanel->SetButtonLabel(B_CANCEL_BUTTON, TCANCEL);
	filepanel->Show();
	
	entry->Unset();
	delete entry;
}

void Application1::LoadProject(BEntry *entry) {
	BFile 	*file=new BFile(entry, B_READ_ONLY);
	BPath	path;
	char	projectname[B_FILE_NAME_LENGTH];
	audioBlock	*audio_block;
	
	entry->GetPath(&path);
	if (file->Read((void *)currentproj, sizeof(projectStruct))==sizeof(projectStruct)) {
		current_folder.SetTo(currentproj->_currentpath);
		if (current_folder[current_folder.Length()-1]!='/') current_folder.Append("/");
		base_folder.SetTo(currentproj->_basepath);
		if (base_folder[base_folder.Length()-1]!='/') base_folder.Append("/");
	
		has_project_changed=false;
		BDirectory HeliosDir;
		BString bf=base_folder;
		bf.RemoveLast("/");
		if (HeliosDir.SetTo(base_folder.String())!=B_OK)
			HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
		HeliosDir.Unset();
		
		window1->Lock();
		settingswindow->Lock();
		window1->view1->SetFilesystem(currentproj->_filesystem);
		window1->view1->SetVolumeName(currentproj->_volumename);
		window1->view1->CDTypeM->ItemAt(currentproj->_cdtype)->SetMarked(true);
		Update_DATA_AUDIO();
		settingswindow->publisherTC->SetText(currentproj->_publisher);
		settingswindow->preparerTC->SetText(currentproj->_preparer);
		settingswindow->applicationTC->SetText(currentproj->_application);
		settingswindow->Unlock();
		entry->GetName(projectname);
		BString *s=new BString(TWINDOWTITLE);
		s->Append(" | ");
		s->Append(projectname);
		window1->SetTitle(s->String());
		delete s;
		
		audio_block=(audioBlock *)malloc(sizeof(audioBlock));
		if (currentproj->_itemBlocksFollowing>0) {
			window1->view1->listview1->iMakeEmpty();
			for (int index=0; index<currentproj->_itemBlocksFollowing;index++) {
				// read the next audio block
				file->Read((void *)audio_block, sizeof(audioBlock));
				AudioRow *item=new AudioRow(index+1, audio_block->_filename, audio_block->_pause, audio_block->_length, audio_block->_length/176400);
				// set all needed audio_block-vars
				if (item!=NULL) {
					item->SetCDTitle(strdup(audio_block->_cdtext_title));
					item->SetIndexList(strdup(audio_block->_indexList));
				}
				window1->view1->listview1->AddRow(item);
			}
		}
		free(audio_block);

		window1->Unlock();

		sprintf(settings._lastprojectfile,"%s",path.Path());
		AddToRecent(new BString(path.Path()));
		ShowRecentProjects();
		currentproject=path.Path();
//		listview_content.MakeEmpty();
//		if (listview_content.Unflatten(file)==B_OK) {
//			window1->Lock();
//			window1->view1->listview1->Instantiate(&listview_content);
//			window1->Unlock();
//		}
	}
	
	file->Unset();
	delete file;
}


void Application1::SaveProject(BEntry *entry) {
	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	BPath path;
	char	projectname[B_FILE_NAME_LENGTH];
	audioBlock	*audio_block;
	
	entry->GetName(projectname);
	

	// set MIME type
	BNode *node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	
	nodeinfo->SetType("application/x-vnd.Helios-project");

	node->Unset();
	delete nodeinfo;
	delete node;
	
	
	// rename the base folder for the newly saved project
	// and set the base_folder-variable to the new path
	BString bf=base_folder;
	bf.RemoveLast("/");
	BEntry *basef=new BEntry(bf.String(), false);
	BString temp=projectname;
	temp.Prepend("Folder_of_");
	basef->Rename(temp.String(), true);
	basef->GetPath(&path);
	base_folder=path.Path();
	if (base_folder[base_folder.Length()-1]!='/') base_folder.Append("/");
	basef->Unset();
	delete basef;
	current_folder=base_folder;

	settingswindow->LockLooper();
	sprintf(currentproj->_currentpath,"%s",base_folder.String());
	sprintf(currentproj->_basepath,"%s",base_folder.String());
	
	currentproj->_filesystem=window1->view1->GetFilesystem();

	
	currentproj->_cdtype=window1->view1->CDTypeM->IndexOf(window1->view1->CDTypeM->FindMarked());
	sprintf(currentproj->_publisher,"%s",settingswindow->publisherTC->Text());
	sprintf(currentproj->_preparer,"%s",settingswindow->preparerTC->Text());
	sprintf(currentproj->_application,"%s",settingswindow->applicationTC->Text());
	sprintf(currentproj->_volumename,"%s",window1->view1->GetVolumeName());
	entry->GetPath(&path);
	sprintf(settings._lastprojectfile,"%s",path.Path());
		AddToRecent(new BString(path.Path()));
		ShowRecentProjects();
	currentproject=path.Path();
	// if project is an audio-cd
	// add audio-item blocks for information on each audio track
	if (window1->view1->CDType()==AUDIOCD_INDEX) {
		currentproj->_itemBlocksFollowing=window1->view1->listview1->iCountItems();
	} else currentproj->_itemBlocksFollowing=0;
	
	// write standard project settings (without audio blocks)
	file->Write((void *)currentproj, sizeof(projectStruct));
	
	// now write audio blocks (must be at the end of the file)
	audio_block=(audioBlock *)malloc(sizeof(audioBlock));
	if (currentproj->_itemBlocksFollowing>0)
		for (int index=0; index<currentproj->_itemBlocksFollowing;index++) {
			AudioRow	*item=(AudioRow *)window1->view1->listview1->RowAt(index);
			
			// set all needed audio_block-vars
			sprintf(audio_block->_cdtext_title,"%s", item->GetCDTitle());
			audio_block->_pause=item->GetPregap();
			sprintf(audio_block->_indexList,"%s", item->GetIndexList());
			sprintf(audio_block->_filename,"%s", item->GetFilename());
			audio_block->_length=item->GetBytes();
			// new write them to disk
			file->Write((void *)audio_block, sizeof(audioBlock));
		}
	free(audio_block);
	
	
	
	// write information to be able to restore the original listview
	// on next startup/load
//	window1->Lock();
//	listview_content.MakeEmpty();
//	if (window1->view1->listview1->Archive(&listview_content)==B_OK)
//		listview_content.Flatten(file);
//	window1->Unlock();
	
	file->Unset();
	settingswindow->UnlockLooper();
	delete file;
		window1->Lock();
		entry->GetName(projectname);
		BString *s=new BString(TWINDOWTITLE);
		s->Append(" | ");
		s->Append(projectname);
		window1->SetTitle(s->String());
		delete s;
		window1->Unlock();
	has_project_changed=false;

}




void Application1::NewProject() {
		
		settingswindow->Lock();
		settingswindow->publisherTC->SetText(settings._s_publisher);
		settingswindow->preparerTC->SetText(settings._s_preparer);
		settingswindow->applicationTC->SetText(settings._s_application);
		//settingswindow->heliospathTC->SetText(base_folder.String());
		settingswindow->Unlock();
		window1->Lock();
		window1->view1->SetVolumeName(settings._s_volumename);
		window1->view1->SetCDType(settings._s_cdtype);
		window1->view1->SetFilesystem(settings._s_filesystem);

		base_folder.SetTo(settings._heliospath).Append(STANDARDPROJECTPATH);
		current_folder=base_folder;

		BDirectory HeliosDir;
		BString bf=base_folder;
		bf.RemoveLast("/");
		if (HeliosDir.SetTo(bf.String())!=B_OK)
			HeliosDir.CreateDirectory(bf.String(),&HeliosDir);
		DeleteAllIn(HeliosDir);
		HeliosDir.Unset();


		sprintf(currentproj->_currentpath,"%s",base_folder.String());
		sprintf(currentproj->_basepath,"%s",base_folder.String());
		currentproj->_filesystem=settings._s_filesystem;
		currentproj->_cdtype=settings._s_cdtype;
		sprintf(currentproj->_publisher,"%s",settings._s_publisher);
		sprintf(currentproj->_preparer,"%s",settings._s_preparer);
		sprintf(currentproj->_application,"%s",settings._s_application);
		sprintf(currentproj->_volumename,"%s",settings._s_volumename);



		has_project_changed=true;
		window1->SetTitle(TWINDOWTITLE);
		Update_DATA_AUDIO();
		window1->Unlock();
}



void Application1::SetAsStandardProject() {
	sprintf(settings._s_publisher,"%s",settingswindow->publisherTC->Text());
	sprintf(settings._s_preparer,"%s",settingswindow->preparerTC->Text());
	sprintf(settings._s_application,"%s",settingswindow->applicationTC->Text());
	sprintf(settings._s_volumename,"%s",window1->view1->GetVolumeName());
	settings._s_cdtype=window1->view1->CDTypeM->IndexOf(window1->view1->CDTypeM->FindMarked());
	settings._s_filesystem=window1->view1->GetFilesystem();
	sprintf(settings._s_currentpath,"%s%s",settings._heliospath, STANDARDPROJECTPATH);
	sprintf(settings._s_basepath,"%s%s",settings._heliospath, STANDARDPROJECTPATH);
}




status_t Application1::GetMultisessionSectors(int32 &start, int32 &end) {
		int arguments=0;
		thread_id Thread;
		const char **args;
		BString str="";
		BString device="dev=";
		BString endstr="";
		int in, out, err;
	
		//if (WaitForDisc(devicepaths[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())])==B_TIMEOUT) return B_TIMEOUT;
		
		device << devices[settingswindow->writerPUM->IndexOf(settingswindow->writerPUM->FindMarked())];
		args = (const char **)malloc(sizeof(char *) * (22));
		args[arguments++] = strdup("/boot/home/config/bin/cdrecord");
		args[arguments++] = strdup(device.String());
		args[arguments++] = strdup("-msinfo");
		args[arguments]=NULL;
		
		printArguments("cdrecord multisessioninfo", args);

		Thread=pipe_command(arguments, args, in, out, err);
		set_thread_priority(Thread,cdrecord_priority);
			if (Thread<0)
				if (window1->Alert("Failure at: load_image(cdrecord)\n\nUnknown load error. Sorry.")==true) 
						 {free(args); return B_ERROR;}
		rename_thread(Thread,CDRECORDTHREADNAME);
		resume_thread(Thread);
		
		size_t bytesread=0;
		char buffer[10000];
		BString s, startsec, endsec;
		
		end=-1;
		start=0;
		while (Thread==find_thread(CDRECORDTHREADNAME)) {
			bytesread=read(out,(void *)buffer, 10000);
			buffer[bytesread]=0;
			if (bytesread>0) {

				// get start and end sector from stdout of cdrecord
				if (strstr(buffer,",")) {
					s=buffer;
					startsec=s.Remove(s.FindFirst(","), s.Length()-s.FindFirst(","));
					start=atol(startsec.String());
					s=buffer;
					endsec=s.Remove(0,s.FindFirst(",")+1);
					end=atol(endsec.String());
					kill_thread(Thread);
				} 

			}
			snooze(10000);
		}
		close(in);close(out);close(err);

		free(args);
		if (end>=start)
		return B_OK; else
		return B_ERROR;
}


void Application1::ShowExportPanel(BMessage *msg) {
	
	BEntry		*entry=new BEntry(settingswindow->heliospathTC->Text(), true);
	entry_ref	ref;
	
	entry->GetRef(&ref);
	if (filepanel!=NULL) delete filepanel;
	filepanel=new BFilePanel(B_SAVE_PANEL, &be_app_messenger, new entry_ref(ref),
				0, false, msg);
	filepanel->SetButtonLabel(B_DEFAULT_BUTTON, C_EXPORT);
	filepanel->SetButtonLabel(B_CANCEL_BUTTON, TCANCEL);
	filepanel->Show();
	
	entry->Unset();
	delete entry;
}



void Application1::ExportAsM3U(BEntry *entry) {

	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	M3UList *list=new M3UList();
	BString path="";
	BEntry	*musicfile=new BEntry();
	BPath	pathtomusicfile;
	AudioRow	*ai;
	BString		filenumber;
	BString		title;
	BString		time;
	int32		count=0;
	
	list->AddItem("[playlist]");
	window1->Lock();
	if (window1->view1->listview1->iCountItems()>0) {
		for (int32 index=0; index<window1->view1->listview1->iCountItems(); index++) {
			ai=(AudioRow *)window1->view1->listview1->RowAt(index);
			
			if ((!ai->IsSplitted()) || (ai->GetSplitIndex()==0)) {
				// create file number entry
				// like this: "File3"
				count++;
				filenumber="File";
				filenumber << count << "=";
				path.SetTo(ai->GetFilename()).Prepend(base_folder);
				musicfile->SetTo(path.String(),true);
				musicfile->GetPath(&pathtomusicfile);
				// generate Title entry
				title="Title=";
				title.Append(ai->GetFilename());
				title.RemoveLast(".mp3");
				title.RemoveLast(".wav");
				title.RemoveLast(".aif");
				title.RemoveLast(".aiff");
				title.RemoveLast(".WAV");
				title.RemoveLast(".MP3");
				title.RemoveLast(".AIFF");
				title.RemoveLast(".AIF");
				title.RemoveLast(".ogg");
				title.RemoveLast(".OGG");
				// generate Time-line if track is not splitted
				if (!(ai->GetSplitIndex()==0)) {
					time="Time=";
					time << (int)ai->GetLength();
					list->AddItem(new BString(time));
				}
				
				list->AddItem(new BString(title));
				filenumber.Append(pathtomusicfile.Path());
				list->AddItem(new BString(filenumber));
			}
		}
		title.SetTo("NumberOfEntries=");
		title << count;
		list->AddItem(new BString(title));
		list->SaveToFile(entry);
	}
	BNode		*node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	nodeinfo->SetType("text/x-playlist");
	delete nodeinfo;
	delete node;
	file->Unset();
	delete file;
	delete musicfile;
	delete list;
	window1->Unlock();
}


// APML is the APlayer Playlist file format (APlayer Module List)
void Application1::ExportAsAPML(BEntry *entry) {

	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	M3UList *list=new M3UList();
	BString path="";
	BEntry	*musicfile=new BEntry();
	BPath	pathtomusicfile;
	AudioRow	*ai;
	BString		filenumber;
	BString		title;
	BString		time;
	BString		lastpath=":";
	BDirectory	dir;
	
	list->AddItem("@*ApML*@");
	window1->Lock();
	if (window1->view1->listview1->iCountItems()>0) {
		for (int32 index=0; index<window1->view1->listview1->iCountItems(); index++) {
			ai=(AudioRow *)window1->view1->listview1->RowAt(index);
			
			// path
			path.SetTo(ai->GetFilename()).Prepend(base_folder);
			musicfile->SetTo(path.String(),true);
			musicfile->GetParent(&dir);
			dir.GetEntry(musicfile);
			musicfile->GetPath(&pathtomusicfile);
			if (lastpath.Compare(pathtomusicfile.Path())!=0) {
				list->AddItem("");
				list->AddItem("@*Path*@");
				lastpath=pathtomusicfile.Path();
				if (lastpath[lastpath.Length()-1]=='/') lastpath.RemoveLast("/");
				list->AddItem(new BString(lastpath));
				list->AddItem("");
				list->AddItem("@*Names*@");
			}
			if ((!ai->IsSplitted()) || (ai->GetSplitIndex()==0))
				list->AddItem(new BString(ai->GetFilename()));
		}
		list->SaveToFile(entry);
	}
	BNode		*node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	nodeinfo->SetType("text/x-aplayer-playlist");
	delete nodeinfo;
	delete node;
	file->Unset();
	delete file;
	delete musicfile;
	delete list;
	window1->Unlock();
}



void Application1::GetEntryNamesIn(BDirectory dir, M3UList *list) {

	int32 c=dir.CountEntries();
	BEntry entry;
	BPath	path;
	
	if (c>0)
	for (int32 i=0; i<c; i++) {
		dir.GetNextEntry(&entry, true);
		if (entry.IsDirectory())
			{
			GetEntryNamesIn(BDirectory(&entry), list);
			} else {
				entry.GetPath(&path);
				list->AddItem(new BString(path.Path()));
			}
	}
	path.Unset();
	entry.Unset();
	
}



void Application1::ExportAsFileList(BEntry *entry) {

	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	M3UList *list=new M3UList();
	BEntry	*musicfile=new BEntry();
	BString		lastpath=base_folder;
	BDirectory	dir;
	
	if (lastpath[lastpath.Length()-1]=='/') lastpath.RemoveLast("/");
	dir.SetTo(lastpath.String());
	GetEntryNamesIn(dir, list);

	list->SaveToFile(entry);

	BNode		*node=new BNode(entry);
	BNodeInfo	*nodeinfo=new BNodeInfo(node);
	nodeinfo->SetType("text/plain");
	file->Unset();
	delete nodeinfo;
	delete node;
	delete file;
	delete musicfile;
	delete list;
}




bool Application1::CheckCDSize() {

	int64 	maxsize=80;
	int	blocksize=2352;
	int64	maxsize_bytes;
	
	if (window1->view1->CDType()==DATACD_INDEX) blocksize=2048;
	if (settingswindow->allowoverburningCB->Value()==B_CONTROL_ON)
		maxsize=90;
		
	maxsize_bytes=(int64)((maxsize*60*176400*(int64)blocksize)/2352);
	
	if (window1->size>maxsize_bytes) {
		BAlert *alert=new BAlert("project too big",TOVERSIZED,TOK);
		alert->Go();
		return false;
	}
	
	return true;
}


void Application1::ExportAsInf(BEntry *entry, int32 index) {

	M3UList *list=new M3UList();
	AudioRow	*ai;
	BString		title;
	
	BFile 	*file=new BFile(entry, B_READ_WRITE | B_ERASE_FILE | B_CREATE_FILE);
	file->Unset();
	delete file;
	list->AddItem("# Audio info file. Created by Helios CDRECORDer.");
	window1->Lock();
			ai=(AudioRow *)window1->view1->listview1->RowAt(index);
				// add performer (not supported by now)
				title="Performer=\t''";
				list->AddItem(strdup(title.String()));
				// add album title
				title="Albumtitle=\t'";
				title.Append(strdup(ai->GetCDTitle()));
				title.Append("'");
				list->AddItem(strdup(title.String()));
				// add album title
				title="Tracktitle=\t'";
				title.Append(strdup(ai->GetCDTitle()));
				title.Append("'");
				list->AddItem(strdup(title.String()));
				// add track number
				title="Tracknumber=\t";
				title << index+1;
				//list->AddItem(strdup(title.String()));
			
				list->SaveToFile(entry);

	delete list;
	window1->Unlock();
}

// creates a filename with serial numbers which are for temporarily use only
const char *Application1::GenerateInfFilename(int32 index) {

	BString	basename="___HELIOS___TRACK_";
	BString	pathtoinf=base_folder;
	BString	final="";
	

	basename=((AudioRow *)(window1->view1->listview1->RowAt(index)))->GetFilename();
	basename.RemoveLast(".mp3");
	basename.RemoveLast(".wav");
	basename.RemoveLast(".MP3");
	basename.RemoveLast(".WAV");
	basename.RemoveLast(".AIFF");
	basename.RemoveLast(".AIF");
	basename.RemoveLast(".aiff");
	basename.RemoveLast(".aif");
	basename.RemoveLast(".ogg");
	basename.RemoveLast(".OGG");
	basename.RemoveLast(".mpeg");
	basename.RemoveLast(".MPEG");
	basename.RemoveLast(".mpeg3");
	basename.RemoveLast(".MPEG3");
	basename.RemoveLast(".mpeg1");
	basename.RemoveLast(".MPEG1");
	if (pathtoinf[pathtoinf.Length()-1]!='/') pathtoinf.Append("/");
	basename << ".inf";
		
	final.SetTo(pathtoinf).Append(basename);
	printf("HELIOS: >%s<\n",final.String());
	return final.String();
}

// writes info files for all tracks kept by the current project
void Application1::SaveCDTEXTAlbum() {
	
	window1->Lock();
	for (int32 index=0; index<window1->view1->listview1->iCountItems();index++) {
		// call export routine
		ExportAsInf(new BEntry(GenerateInfFilename(index), true), index);
	}	
	window1->Unlock();	
}


// remove the temporary files (info files for audio-cd burning)
// located in the temp-path of helios
void Application1::RemoveInfFiles() {
	
	BEntry	*entry=new BEntry();
	
	window1->Lock();
	for (int32 index=0; index<window1->view1->listview1->iCountItems();index++) {
		entry->SetTo(GenerateInfFilename(index),true);
		if (entry->Exists())
			entry->Remove();
		entry->Unset();
	}	
	delete entry;
	window1->Unlock();	
}

BString SubString(BString *string, int32 first, int32 last) {
	BString temp;
	
	temp.SetTo(string->String());
	temp.Remove(last,temp.Length()-last);
	temp.Remove(0, first);
	return temp;
}

void Application1::PopulateFoldersMenu() {
	BString current=current_folder;
	current.RemoveLast("/");
	BitmapMenuItem	*mi;
	BMessage	*msg;
	BEntry		entry;
	entry_ref	ref;
	
	window1->Lock();
	window1->view1->listview1->statusView->RemoveMenuItems();
	if (current.Length()>base_folder.Length()) 
		current.Remove(current.FindLast("/"), current.Length()-current.FindLast("/"));

	while (current.Length()>base_folder.Length()) {
		msg=new BMessage(FOLDERMENU_MSG);
		msg->AddString("folder", current);
		if (entry.SetTo(current.String(), true)==B_OK) {
			entry.GetRef(&ref);
			entry.Unset();
		}
		mi=new BitmapMenuItem(SubString(&current, current.FindLast("/")+1, current.Length()).String(), GetIcon(&ref), msg);
		window1->view1->listview1->statusView->AddMenuItem(mi);
		current.Remove(current.FindLast("/"), current.Length()-current.FindLast("/"));
	}
	current=current_folder;
	if (current.Length()>base_folder.Length()) {
		BString t="";
		if (window1->view1->CDType()==DATACD_INDEX) t=window1->view1->GetVolumeName();
		if (window1->view1->CDType()==BOOTABLECD_INDEX) t=window1->view1->GetVolumeNameBootCD();
		msg=new BMessage(FOLDERMENU_MSG);
		msg->AddString("folder", base_folder);
		entry.SetTo(base_folder.String(), true);
		entry.GetRef(&ref);
		entry.Unset();
		mi=new BitmapMenuItem(t.String(), GetIcon(&ref),  msg);
		window1->view1->listview1->statusView->AddMenuItem(mi);
	}
	window1->Unlock();
}