#include "AudioRow.h"
#include "Defines.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>


AudioRow::AudioRow(int32 track, const char *text, int32 pause, off_t size, off_t length):BRow(){

	trackFIELD=new BIntegerField(track);
	pregapFIELD=new BStringField(Format(pause));
	nameFIELD=new BStringField(text);
	lengthFIELD=new BStringField(Format(length));
	bytesFIELD=new BSizeField(size);
	indexFIELD=new BStringField("0.00s");
	cdtextFIELD=new BStringField("");
	SetField(trackFIELD, TRACKNUMBER_FIELD);
	SetField(pregapFIELD, PREGAP_FIELD);
	SetField(nameFIELD, TRACKNAME_FIELD);
	SetField(lengthFIELD, LENGTH_FIELD);
	SetField(bytesFIELD, BYTES_FIELD);
	SetField(indexFIELD, INDEX_FIELD);
	SetField(cdtextFIELD, CDTEXT_FIELD);

	splitindex=0;
	SetSplitted(false);
	SetPregap(pause);
	SetLength(length);
	SetTrackNumber(track);
	kListc=1;
}

//AudioRow::AudioRow(int32 track, const char *text, int32 pause, off_t size):BRow() {
//	AudioRow(track, text, 0, size, size*176400);
//}

AudioRow::~AudioRow() {
}

const char *AudioRow::Format(off_t seconds) {
	int m, s;
	BString temp="";
	BString s_nought="";
	static char t[20];
	
	m=seconds/60;
	s=(seconds-(m*60));
	if (s<10) s_nought="0";
	sprintf(t, "%dm %s%ds", m, s_nought.String(),s);
//	temp << m << "m " << s_nought.String() << s << "s";
	return (const char *)&t;	
}


void AudioRow::SetBytes(off_t bytes) {
	bytesFIELD->SetSize(bytes);
}

off_t AudioRow::GetBytes() {
	return bytesFIELD->Size();
}

void AudioRow::SetLength(off_t seconds) {
	lengthFIELD->SetString(Format(seconds));
	tracklength=seconds;
}

off_t AudioRow::GetLength() {
	return tracklength;
}

void AudioRow::SetPregap(int32 frames) {
	pregapFIELD->SetString(Format(frames));
	pregapframes=frames;
}

int32 AudioRow::GetPregap() {
	return pregapframes;
}

void AudioRow::SetFilename(const char *leaf) {
	nameFIELD->SetString(leaf);
}

const char *AudioRow::GetFilename() {
	return nameFIELD->String();
}

bool AudioRow::IsSplitted() {
	return splitted;
}

void AudioRow::SetSplitted(bool splitted) {
	this->splitted=splitted;
}

int32 AudioRow::GetSplitIndex() {
	return splitindex;
}

void AudioRow::SetSplitIndex(int32 index) {
	splitindex=index;
}

bool AudioRow::IsListCorrect() {
	int32 last;
	
	if (kListc==1) {kList[0]=0;return true;}
	if (kListc<1) return false;
	last=kList[0];
	for (int i=1; i<kListc;i++) {
		if ((last>kList[i]) || (GetLength()*75.0<kList[i])) return false;
		last=kList[i];
	}
	return true;
}

void AudioRow::SetIndexList(const char *list) {
	int i=0;
	BString s, l=list;
	int32 f=0;
	
	kLastList=kList;
	l.ReplaceAll(",,",",");
	l.RemoveAll(" ");
	if (l.Length()==0) {
		l="0.00s,";
		kList[0]=0;
		kListc=1;
	}
	if (l[l.Length()-1]!=',') l.Append(",");
	s=l;
	while (s.FindFirst(",")!=B_ERROR) {
		s.Truncate(s.FindFirst(","));
		//printf("%d:%s\n",i+1,s.String());
		if (s.FindFirst("f")!=B_ERROR) {
			s.RemoveFirst("f");
			if (s.Length()>0)
			f=atoi(s.String());
		} else
		if (s.Length()>0) {
			s.RemoveFirst("s");
			if (s.Length()>0)
			f=(int32)(atof(s.String())*75.0);
		}
		kList[i++]=f;
		l.Remove(0,l.FindFirst(",")+1);
		s=l;
	}
	kListc=i;
	kList[0]=0;
	if (!IsListCorrect()) kList=kLastList;
	
}

const char *AudioRow::GetIndexList() {
	BString list="";
	char *list2="";
	for (int i=0; i<kListc; i++) {
		asprintf(&list2,"%0.2fs, ",(float)kList[i]/75.0);
		list.Append(list2);
	}
	return list.String();
}

const char *AudioRow::GetCdrecordReadyList() {
	BString list="";
	char list2[20];
	
	for (int i=0; i<kListc; i++) {
		sprintf(list2,"%ld,",kList[i]);
		list.Append(list2);
	}
	if (list.Length()==0) list.Append("0");
		if (list[list.Length()-1]==',') 
			list.RemoveLast(",");
	return (const char *)strdup(list.String());
}

const char *AudioRow::GetCDTitle() {
	return cdtextFIELD->String();
}

void AudioRow::SetCDTitle(const char *title) {
	cdtextFIELD->SetString(title);
}

void AudioRow::SetTrackNumber(int32 track) {
	trackFIELD->SetValue(track);
}

int32 AudioRow::GetTrackNumber() {
	return trackFIELD->Value();
}







