#include "BootableCDPanel.h"
#include "Stringtable.h"
#include "Defines.h"
#include <Application.h>
#include "FolderRow.h"
#include "Application1.h"
#include "ListView1.h"

BootableCDPanel::BootableCDPanel(BRect frame):BView(frame, "DataCDPanel", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW) {
	filesystemPUM=new BPopUpMenu("");
	filesystemMF=new BMenuField(BRect(0,0,200,21),"filesystemMF", TFILESYSTEM, filesystemPUM, false);
	filesystemMF->SetDivider(StringWidth(TFILESYSTEM)+10);
	AddChild(filesystemMF);
	
	iso9660MI=new BMenuItem("ISO 9660", NULL);
	filesystemPUM->AddItem(iso9660MI);
	iso9660longMI=new BMenuItem("ISO 9660 long", NULL);
	filesystemPUM->AddItem(iso9660longMI);
	bfsMI=new BMenuItem("BeOS bfs", NULL);
	filesystemPUM->AddItem(bfsMI);
	hfsMI=new BMenuItem("MacOS hfs", NULL);
	filesystemPUM->AddItem(hfsMI);
	rockridgeMI=new BMenuItem("Unix RockRidge", NULL);
	filesystemPUM->AddItem(rockridgeMI);
	jolietMI=new BMenuItem("Windows Joliet", NULL);
	filesystemPUM->AddItem(jolietMI);
	
	volumenameTC=new BTextControl(BRect(0,0,200,21), "volumenameTC", TVOLUMENAME, TUNTITLED, new BMessage('mm05'));
	volumenameTC->SetAlignment(B_ALIGN_LEFT, B_ALIGN_LEFT);
	volumenameTC->SetDivider(StringWidth(TVOLUMENAME)+28);
	volumenameTC->TextView()->SetMaxBytes(32);
	volumenameTC->SetModificationMessage(new BMessage('mm05'));
	volumenameTC->SetTarget(be_app_messenger);
	AddChild(volumenameTC);
	eltoritoimageTC=new BTextControl(BRect(0,0,200,21), "eltoritoimageTC", TELTORITOIMAGE, "", NULL);
	eltoritoimageTC->SetDivider(StringWidth(TVOLUMENAME)+28);
	AddChild(eltoritoimageTC);
	SetViewColor((rgb_color){216,216,216,255});
}

BootableCDPanel::~BootableCDPanel() {
	delete filesystemMF;
	delete volumenameTC;
	delete eltoritoimageTC;
}

void BootableCDPanel::AllAttached() {
	filesystemMF->SetDivider(StringWidth(TVOLUMENAME)+30);
	filesystemMF->MoveTo(5,4);
	filesystemMF->ResizeTo(200, 22);
	volumenameTC->MoveTo(5,29);
	volumenameTC->ResizeTo(200, 20);
	eltoritoimageTC->MoveTo(5, 53);
	eltoritoimageTC->ResizeTo(200,20);
}

const char	*BootableCDPanel::GetFileSystem() {
	switch (filesystemPUM->IndexOf(filesystemPUM->FindMarked())) {
		// ISO 9660
		case 0:	return ""; break;
		// ISO 9660 long
		case 1:	return "-l"; break;
		// BeOS bfs
		case 2:	return "-bfs ;-)"; break;
		// MacOS hfs
		case 3:	return "-h"; break;
		// Unix RockRidge
		case 4:	return ""; break;
		// Windows Joliet
		case 5:	return "-J"; break;
		default:	return "ERROR"; break;
	}
	return "ERROR";
}

const char *BootableCDPanel::GetVolumeName() {
	return volumenameTC->Text();
}

void	BootableCDPanel::SetVolumeName(const char *name) {
	volumenameTC->SetText(name);
}

const char *BootableCDPanel::GetImageName() {
	return eltoritoimageTC->Text();
}

void	BootableCDPanel::SetImageName(const char *name) {
	eltoritoimageTC->SetText(name);
}

void	BootableCDPanel::MessageReceived(BMessage *msg) {
	switch(msg->what) {
		case LISTITEM_DROPPED:
				int32	i;
				if (msg->FindInt32("index", &i)==B_OK) {
					BString str;
					const char *str2;
					str.SetTo(((Application1 *)be_app)->current_folder.String());
					str.RemoveFirst(((Application1 *)be_app)->base_folder.String());
					str.Prepend("/");
					//str.RemoveAll("//");
					//if (str.Length()==0) str="/";
					str2=((FolderRow *)(((Application1 *)be_app)->window1->view1->listview1->RowAt(i)))->GetFilename();
					str.Append(str2);
					SetImageName(str.String());
				}
				break;
		default:	BView::MessageReceived(msg);
				break;
	}
}
