#include "DataCDPanel.h"
#include "Stringtable.h"
#include <Application.h>

DataCDPanel::DataCDPanel(BRect frame):BView(frame, "DataCDPanel", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW) {
	multisessionCB=new BCheckBox(BRect(frame.left+4, frame.top, frame.right-4, frame.top+16), "copyprotection", TMULTISESSION, NULL);
	filesystemPUM=new BPopUpMenu("");
	filesystemMF=new BMenuField(BRect(0,0,200,21),"filesystemMF", TFILESYSTEM, filesystemPUM, false);
	filesystemMF->SetDivider(StringWidth(TFILESYSTEM)+10);
	
	iso9660MI=new BMenuItem("ISO 9660", NULL);
	filesystemPUM->AddItem(iso9660MI);
	iso9660longMI=new BMenuItem("ISO 9660 long", NULL);
	filesystemPUM->AddItem(iso9660longMI);
	bfsMI=new BMenuItem("BeOS bfs", NULL);
	filesystemPUM->AddItem(bfsMI);
	hfsMI=new BMenuItem("MacOS hfs", NULL);
	filesystemPUM->AddItem(hfsMI);
	rockridgeMI=new BMenuItem("Unix RockRidge", NULL);
	filesystemPUM->AddItem(rockridgeMI);
	jolietMI=new BMenuItem("Windows Joliet", NULL);
	filesystemPUM->AddItem(jolietMI);
	
	volumenameTC=new BTextControl(BRect(0,0,200,21), "volumenameTC", TVOLUMENAME, TUNTITLED, new BMessage('mm05'));
	volumenameTC->SetAlignment(B_ALIGN_LEFT, B_ALIGN_LEFT);
	volumenameTC->SetDivider(StringWidth(TVOLUMENAME)+28);
	volumenameTC->TextView()->SetMaxBytes(32);
	volumenameTC->SetModificationMessage(new BMessage('mm05'));
	volumenameTC->SetTarget(be_app_messenger);
	AddChild(filesystemMF);
	AddChild(volumenameTC);
	AddChild(multisessionCB);
	SetViewColor((rgb_color){216,216,216,255});
}

DataCDPanel::~DataCDPanel() {
	delete multisessionCB;
	delete filesystemMF;
	delete volumenameTC;
}

void DataCDPanel::AllAttached() {
	filesystemMF->SetDivider(StringWidth(TVOLUMENAME)+30);
	filesystemMF->MoveTo(5,4);
	filesystemMF->ResizeTo(200, 22);
	volumenameTC->MoveTo(5,29);
	volumenameTC->ResizeTo(200, 20);
	multisessionCB->MoveTo(5,52);
	multisessionCB->ResizeTo(22+StringWidth(multisessionCB->Label()), 16);
}

const char	*DataCDPanel::GetFileSystem() {
	switch (filesystemPUM->IndexOf(filesystemPUM->FindMarked())) {
		// ISO 9660
		case 0:	return ""; break;
		// ISO 9660 long
		case 1:	return "-l"; break;
		// BeOS bfs
		case 2:	return "-bfs ;-)"; break;
		// MacOS hfs
		case 3:	return "-h"; break;
		// Unix RockRidge
		case 4:	return ""; break;
		// Windows Joliet
		case 5:	return "-J"; break;
		default:	return "ERROR"; break;
	}
	return "ERROR";
}

bool	DataCDPanel::IsMultisession() {
	return (multisessionCB->Value()==B_CONTROL_ON);
}

void	DataCDPanel::SetMultisession(bool flag) {
	if (flag) 	multisessionCB->SetValue(B_CONTROL_ON); else
			multisessionCB->SetValue(B_CONTROL_OFF);
}

const char *DataCDPanel::GetVolumeName() {
	return volumenameTC->Text();
}

void	DataCDPanel::SetVolumeName(const char *name) {
	volumenameTC->SetText(name);
}
