#include "ProtocolWindow.h"
#include <String.h>
#include <unistd.h>
#include "Stringtable.h"

#define LOG_CLEAR	'LOGc'

/*
BTextView(BRect frame, const char *name, BRect textRect, 
      uint32 resizingMode, uint32 flags)
*/

ProtocolView::ProtocolView(BRect frame, const char *name, BRect textRect, uint32 resizingMode, uint32 flags, int stream)
		:BTextView(frame, name, textRect, resizingMode, flags) {
	fStream=stream;
}

ProtocolView::~ProtocolView() {
}

void ProtocolView::Pulse() {
	
	if (fStream!=-1) {
		size_t bytesread=0;
		bytesread=read(fStream,(void *)fBuffer, 10000);
		fBuffer[bytesread]=0;
		((ProtocolWindow *)this->Window())->AddText(fBuffer);
	}

	BTextView::Pulse();
}

void ProtocolView::Start(int stream) {
	fStream=stream;
}

int ProtocolView::Stop() {
	int old=fStream;
	fStream=-1;
	return old;
}



ProtocolWindow::ProtocolWindow(BRect frame, const char *title, window_type type, int stream)
			:BWindow(frame, title, type, 0) {
//	contentView=new BTextView(BRect(0,0,frame.Width()-B_V_SCROLL_BAR_WIDTH, frame.Height()-B_H_SCROLL_BAR_HEIGHT), "contentView", frame,B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
//	scrollView=new BScrollView("scrollView", contentView, B_FOLLOW_ALL_SIDES, 0, true, true);
	BMenuBar *mb=new BMenuBar(BRect(0,0,frame.Width(),1), TMI_FILE);
	mainM=new BMenu(TMI_FILE);
	clearMI=new BMenuItem(TMI_CLEAR, new BMessage(LOG_CLEAR), 'C');
	closeMI=new BMenuItem(TMI_CLOSE, new BMessage(B_QUIT_REQUESTED), 'W');
	contentView=new ProtocolView(BRect(0,0,frame.Width()-B_V_SCROLL_BAR_WIDTH, frame.Height()-(B_H_SCROLL_BAR_HEIGHT)), "contentView", BRect(0,0,2000,1000),B_FOLLOW_ALL_SIDES, B_WILL_DRAW, stream);
	scrollView=new BScrollView("scrollView", contentView, B_FOLLOW_ALL_SIDES, 0, true, true);
	SetLook(B_DOCUMENT_WINDOW_LOOK);
	contentView->MakeEditable(false);
	contentView->MakeResizable(false);
	fStream=stream;
	AddChild(scrollView);
	AddChild(mb);
	mb->AddItem(mainM);
	mainM->AddItem(clearMI);
	mainM->AddItem(closeMI);
	MoveTo(40,40);
	float width, height;
	mb->GetPreferredSize(&width,&height);
	scrollView->MoveBy(0,height+1);
	scrollView->ResizeBy(0,-(height+1));
}


bool ProtocolWindow::QuitRequested() {
	while (!IsHidden()) Hide();
	return false;
}

ProtocolWindow::~ProtocolWindow() {
}

void ProtocolWindow::MessageReceived(BMessage *msg) {
	if (msg->what==LOG_CLEAR)
		Clear();
	else
		BWindow::MessageReceived(msg);
}

void ProtocolWindow::Clear() {
	Lock();
	contentView->SetText("", 0L);
	Unlock();
}

void ProtocolWindow::AddText(const char *string) {
	Lock();
	BString *s=new BString(string);
	s->Prepend(contentView->Text());
	s->ReplaceAll("\r", "\n", contentView->TextLength());
	while (s->FindFirst("\b")!=B_ERROR) {
		s->Remove(s->FindFirst("\b")-1, 1);
		s->RemoveFirst("\b");
	}
	
	contentView->SetText(s->String(), s->Length());
	delete s;
	contentView->ScrollToOffset(contentView->TextLength());
	contentView->Sync();
	Unlock();
}

void ProtocolWindow::Start(int stream) {
	contentView->Start(stream);
}

int ProtocolWindow::Stop() {
	return contentView->Stop();
}
