#include "SettingsWindow.h"
#include "Stringtable.h"
#include "Defines.h"
#include <String.h>

SettingsWindow::
	SettingsWindow(BRect frame,const char *title):
		BWindow(frame,title, B_FLOATING_WINDOW,B_NOT_RESIZABLE | B_NOT_ZOOMABLE,B_CURRENT_WORKSPACE) {
		
	SetLook(B_FLOATING_WINDOW_LOOK);
	SetFeel(B_MODAL_APP_WINDOW_FEEL);

	// check the longest string for menufield-labels to get a nicer look
	BString longest=TCDRDRIVE;
	float leftMargin=0;
		
	// create the view
	settingsview=new BView(Bounds(),"settingsview",B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_NAVIGABLE);
	AddChild(settingsview);
	settingsview->SetViewColor((rgb_color) {216,216,216,255});

	if (settingsview->StringWidth(TCDREADERLABEL)>leftMargin) leftMargin=settingsview->StringWidth(TCDREADERLABEL);
	if (settingsview->StringWidth(TCDRDRIVE)>leftMargin) leftMargin=settingsview->StringWidth(TCDRDRIVE);
	if (settingsview->StringWidth(THELIOSPATH)>leftMargin) leftMargin=settingsview->StringWidth(THELIOSPATH);
	if (settingsview->StringWidth(TPUBLISHER)>leftMargin) leftMargin=settingsview->StringWidth(TPUBLISHER);
	if (settingsview->StringWidth(TPREPARER)>leftMargin) leftMargin=settingsview->StringWidth(TPREPARER);
	if (settingsview->StringWidth(TAPPLICATION)>leftMargin) leftMargin=settingsview->StringWidth(TAPPLICATION);
	leftMargin+=8;
	
	// CD Writer MenuField
	writerPUM=new BPopUpMenu("                            ");
	writerMF=new BMenuField(BRect(5,5,248,25),"CD writer field",TCDRDRIVE, writerPUM);
	writerMF->SetDivider(leftMargin);
	settingsview->AddChild(writerMF);

	// CD Reader MenuField
	readerPUM=new BPopUpMenu("                            ");
	readerMF=new BMenuField(BRect(5,30,248,50),"CD reader field",TCDREADERLABEL, readerPUM);
	readerMF->SetDivider(leftMargin);
	settingsview->AddChild(readerMF);
	
	// eject CheckBoxes for each device
	ejectwriterCB=new BCheckBox(BRect(255,7,338,27),"eject writer",TEJECT,NULL);
	settingsview->AddChild(ejectwriterCB);
	ejectreaderCB=new BCheckBox(BRect(255,32,338,52),"eject reader",TEJECT,NULL);
	settingsview->AddChild(ejectreaderCB);

	// writer speed label and slider
	writerspeedSV=new BStringView(BRect(leftMargin+5,50,300,70),"writer speed",TRECORDERSPEED);
	settingsview->AddChild(writerspeedSV);

	writerspeedSL=new BSlider(BRect(leftMargin+5,70,340,95),"writer speed",NULL,new BMessage(BURNSPEED_MSG),1,16,B_BLOCK_THUMB);
	writerspeedSL->SetHashMarks(B_HASH_MARKS_BOTTOM);
	writerspeedSL->SetHashMarkCount(16);
	writerspeedSL->SetLimitLabels("2x","32x");
//	writerspeedSL->SetModificationMessage(new BMessage(BURNSPEED_MSG));
	writerspeedSL->SetTarget(be_app_messenger);
	settingsview->AddChild(writerspeedSL);

	// BurnProof CheckBox
	burnproofCB=new BCheckBox(BRect(5+leftMargin,107,leftMargin+110,122),"BurnProof",TBURNPROOF,NULL);
	settingsview->AddChild(burnproofCB);
	// Simulate CheckBox
	simulateCB=new BCheckBox(BRect(5+leftMargin,127,leftMargin+110,142),"simulate",TSIMULATION,NULL);
	settingsview->AddChild(simulateCB);
	// Streaming Mode CheckBox
	streamingCB=new BCheckBox(BRect(5+leftMargin,147,leftMargin+110,162),"streaming",TSTREAMING,NULL);
	settingsview->AddChild(streamingCB);
	// Fixation CheckBox
	fixateCB=new BCheckBox(BRect(130+5+leftMargin,107,140+leftMargin+110,122),"fixate",TFIXATE,new BMessage(FIXATE_CHKBOX_MSG));
	fixateCB->SetTarget(be_app_messenger);
	settingsview->AddChild(fixateCB);
	// DAO<->TAO CheckBox
	writeDAOCB=new BCheckBox(BRect(130+5+leftMargin,147,140+leftMargin+130,162),"DAO",TDAOMODE,NULL);
	settingsview->AddChild(writeDAOCB);
	// 2-drive-copy
	twodrivecopyCB=new BCheckBox(BRect(5+leftMargin,167,leftMargin+110,182),"2drivecopy",TTWODRIVECOPY,NULL);
	settingsview->AddChild(twodrivecopyCB);
	// allow overburning (oversized discs)
	allowoverburningCB=new BCheckBox(BRect(130+5+leftMargin,167,140+leftMargin+130,182), "oversizing", TALLOWOVERBURNING, NULL);
	settingsview->AddChild(allowoverburningCB);
	
	BFont *TCfont=new BFont(be_fixed_font);
	TCfont->SetSize(11.0);
	// Helios Path
	heliospathTC=new BTextControl(BRect(5,190,340,210),"heliospathTC",THELIOSPATH, "",new BMessage());
	heliospathTC->SetDivider(leftMargin-2);
	heliospathTC->TextView()->SetMaxBytes(300);
	settingsview->AddChild(heliospathTC);
	// CD Publisher
	publisherTC=new BTextControl(BRect(5,213,340,233),"publisherTC",TPUBLISHER, "",NULL);
	publisherTC->SetDivider(leftMargin-2);
	publisherTC->TextView()->SetMaxBytes(128);
	publisherTC->TextView()->SetFontAndColor(TCfont);
	settingsview->AddChild(publisherTC);
	// CD Preparer
	preparerTC=new BTextControl(BRect(5,236,340,256),"preparerTC",TPREPARER, "",NULL);
	preparerTC->SetDivider(leftMargin-2);
	preparerTC->TextView()->SetMaxBytes(128);
	preparerTC->TextView()->SetFontAndColor(TCfont);
	settingsview->AddChild(preparerTC);
	// CD Application
	applicationTC=new BTextControl(BRect(5,259,340,279),"applicationTC",TAPPLICATION, "",NULL);
	applicationTC->SetDivider(leftMargin-2);
	applicationTC->TextView()->SetMaxBytes(128);
	applicationTC->TextView()->SetFontAndColor(TCfont);
	settingsview->AddChild(applicationTC);
}


SettingsWindow::~SettingsWindow() {
}

void SettingsWindow::Show(void) {

	SetWorkspaces(B_CURRENT_WORKSPACE);
	BWindow::Show();
}

void SettingsWindow::Hide(void) {

	be_app->PostMessage('mm04');
	BWindow::Hide();
}

bool
SettingsWindow::QuitRequested() {

	Hide();
//	((Application1 *)be_app)->SaveSettings();
	return false;
}

